#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#
#    Copyright (C) 2017, Kai Raphahn <kai.raphahn@laburec.de>
#

import sqlite3
from unittest import mock as mock

from bbutil.database import SQLite, Types, Table, Data

__all__ = [
    "sqlite_unknown_error",
    "sqlite_operational_error",
    "sqlite_integrity_error",
    "mock_operational_error",

    "get_sqlite_operational_error",
    "get_sqlite_integrity_error",
    "get_sqlite_return_false",

    "get_table_01",
    "get_data_01",
    "get_data_02",
    "get_data_03",
    "get_data_04",
    "get_data_05",
    "get_data_06",
    "get_data_07",
    "get_data_08"
]

sqlite_unknown_error = Exception("Something strange did happen!")
sqlite_operational_error = sqlite3.OperationalError('This did go boing!!')
sqlite_integrity_error = sqlite3.IntegrityError('These values did go boing!!')
mock_operational_error = mock.Mock(side_effect=sqlite_operational_error)


def get_sqlite_operational_error():
    _cursor = mock.Mock(name="SQLite3.cursor")
    _cursor.execute = mock.Mock(side_effect=sqlite_operational_error)

    _connection = mock.Mock(name="SQLite3.connect")
    _connection.cursor = mock.Mock(return_value=_cursor)

    _sqlite = mock.Mock(name="SQLite3", return_value=_connection)
    return _sqlite


def get_sqlite_integrity_error():
    _cursor = mock.Mock(name="SQLite3.cursor")
    _cursor.execute = mock.Mock(side_effect=sqlite_integrity_error)

    _connection = mock.Mock(name="SQLite3.connect")
    _connection.cursor = mock.Mock(return_value=_cursor)

    _sqlite = mock.Mock(name="SQLite3", return_value=_connection)
    return _sqlite


def get_sqlite_return_false():
    _sqlite = mock.Mock(name="close", return_value=False)
    return _sqlite


def get_table_01(sqlite_object: SQLite) -> Table:
    _table = Table(name="tester01", sqlite=sqlite_object)
    _table.add_column(name="testid", data_type=Types.integer, unique=True)
    _table.add_column(name="use_test", data_type=Types.bool)
    _table.add_column(name="testname", data_type=Types.string)
    _table.add_column(name="path", data_type=Types.string)
    return _table


def get_data_01() -> Data:
    _names = [
        "testid",
        "use_test",
        "testname",
        "path"
    ]

    _values = [
        1,
        True,
        "Test01",
        "testers/"
    ]

    _data = Data(_names, _values)
    return _data


def get_data_02() -> Data:
    _names = [
        "testidx",
        "use_test",
        "testname",
        "path"
    ]

    _values = [
        1,
        True,
        "Test01",
        "testers/"
    ]

    _data = Data(_names, _values)
    return _data


def get_data_03() -> Data:
    _names = [
        "testid",
        "use_test",
        "testname",
        "path"
    ]

    _values = [
        15235670141346654134,
        True,
        "Test01",
        "testers/"
    ]

    _data = Data(_names, _values)
    return _data


def get_data_04() -> Data:
    _names = [
        "testid",
        "use_test",
        "testname",
        "path"
    ]

    _values = [
        1,
        True,
        "Test01",
        [
            "List1",
            "List2"
        ]
    ]

    _data = Data(_names, _values)
    return _data


def get_data_05() -> list:
    _names = [
        "testid",
        "use_test",
        "testname",
        "path"
    ]

    _values = [
        [1, True, "Test01", "testers/"],
        [2, True, "Test02", "testers/"],
        [3, True, "Test03", "testers/"],
        [4, False, "Test04", "testXXs/"],
        [5, True, "Test05", "testXXs/"],
        [6, True, "Test06", "testXXs/"]
    ]

    _datas = []

    for _value in _values:
        _data = Data(_names, _value)
        _datas.append(_data)
    return _datas


def get_data_06() -> list:
    _names = [
        "testid",
        "use_test",
        "testname",
        "path"
    ]

    _values = [
        [1, True, "Test01", "testers/"],
        [2, True, "Test02", "testers/"],
        [3, True, "Test03", "testers/"],
        [4, False, "Test04"],
        [5, True, "Test05", "testXXs/"],
        [6, True, "Test06", "testXXs/"]
    ]

    _datas = []

    for _value in _values:
        _data = Data(_names, _value)
        _datas.append(_data)
    return _datas


def get_data_07() -> Data:
    _names = [
        "testid",
        "use_test",
        "testname",
        "path"
    ]

    _values = [4, True, "Test04", "testXXs/"]

    _data = Data(_names, _values)
    return _data


def get_data_08() -> Data:
    _names = [
        "testid",
        "use_test",
        "testname",
        "path"
    ]

    _values = [4, True, "Test04", 15235670141346654134]

    _data = Data(_names, _values)
    return _data



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#
#    Copyright (C) 2017, Kai Raphahn <kai.raphahn@laburec.de>
#

import sys

from unittest.mock import Mock

__all__ = [
    "MockArgumentParser01",
    "MockArgumentParser02",
    "MockArgumentParser03",
    "MockArgumentParser04",
    "MockArgumentParser05",
    "mock_open"
]

mock_open = Mock(side_effect=OSError("BUMM!"))

if sys.platform == "win32":
    _ls = "dir.bat"
else:
    _ls = "/usr/bin/ls"


class _Options01(object):

    def __init__(self):
        self.bla = "/usr/local/bin/bla"
        self.bleb = 10
        self.verbose = 0
        self.ls = _ls
        self.work = "test"
        return


class MockArgumentParser01(object):

    def __init__(self):
        self.add_argument = Mock()
        self.parse_args = Mock(return_value=_Options01())
        return


class _Options02(object):

    def __init__(self):
        self.bla = "/usr/local/bin/bla"
        self.bleb = 10
        self.ls = _ls
        self.work = "test"
        return


class MockArgumentParser02(object):

    def __init__(self):
        self.add_argument = Mock()
        self.parse_args = Mock(return_value=_Options02())
        return


class _Options03(object):

    def __init__(self):
        self.bla = "/usr/local/bin/bla"
        self.bleb = 10
        self.verbose = 0
        self.work = "test"
        return


class MockArgumentParser03(object):

    def __init__(self):
        self.add_argument = Mock()
        self.parse_args = Mock(return_value=_Options03())
        return


class _Options04(object):

    def __init__(self):
        self.bla = "/usr/local/bin/bla"
        self.bleb = 10
        self.verbose = 0
        self.ls = "/usr/toilet/ls"
        self.work = "test"
        return


class MockArgumentParser04(object):

    def __init__(self):
        self.add_argument = Mock()
        self.parse_args = Mock(return_value=_Options04())
        return


class _Options05(object):

    def __init__(self):
        self.bla = "/usr/local/bin/bla"
        self.bleb = 10
        self.verbose = 0
        self.ls = ""
        self.work = "test"
        return


class MockArgumentParser05(object):

    def __init__(self):
        self.add_argument = Mock()
        self.parse_args = Mock(return_value=_Options05())
        return



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#
#    Copyright (C) 2017, Kai Raphahn <kai.raphahn@laburec.de>
#

import os
import sys
import unittest

import unittest.mock as mock

import bbutil
from bbutil.utils import full_path, openjson

from tests.helper import set_log, set_module
from tests.helper.config import (MockArgumentParser01, MockArgumentParser02, MockArgumentParser03,
                                 MockArgumentParser04, MockArgumentParser05, mock_open)

from testdata.app.config import AppConfig

__all__ = [
    "TestConfig"
]

oserror = OSError("Something strange did happen!")
mock_oserror = mock.Mock(side_effect=oserror)
mock_remove = mock.Mock()

if sys.platform == "win32":
    _ls = "dir.bat"
else:
    _ls = "/usr/bin/ls"


class TestConfig(unittest.TestCase):
    """Testing class for locking module."""

    def setUp(self):
        set_log()
        set_module()
        return

    @mock.patch('argparse.ArgumentParser', new=MockArgumentParser01)
    def test_init_01(self):
        self.assertIsNotNone(bbutil.module)

        _work = full_path("{0:s}/test".format(os.getcwd()))
        if os.path.exists(_work) is False:
            os.mkdir(_work)

        _config = AppConfig(use_parser=True)

        _check2 = _config.init()
        self.assertTrue(_check2)
        self.assertTrue(_config.valid)
        self.assertEqual(_config.verbose, 0)
        self.assertEqual(_config.bla, "/usr/local/bin/bla")
        self.assertEqual(_config.bleb, 10)
        self.assertEqual(_config.ls, _ls)
        self.assertEqual(_config.work, _work)
        return

    @mock.patch('argparse.ArgumentParser', new=MockArgumentParser01)
    def test_init_02(self):
        self.assertIsNotNone(bbutil.module)

        _config = AppConfig(use_parser=False)

        _check2 = _config.init()
        self.assertTrue(_check2)
        self.assertTrue(_config.valid)
        return

    @mock.patch('argparse.ArgumentParser', new=MockArgumentParser01)
    def test_init_03(self):
        bbutil.module = None

        # noinspection PyTypeChecker
        bbutil.set_module(None)
        self.assertIsNone(bbutil.module)

        _config = AppConfig(use_parser=True)

        _check2 = _config.init()
        self.assertFalse(_check2)
        self.assertFalse(_config.valid)
        return

    @mock.patch('argparse.ArgumentParser', new=MockArgumentParser02)
    def test_init_04(self):
        self.assertIsNotNone(bbutil.module)

        _config = AppConfig(use_parser=True)

        _check2 = _config.init()
        self.assertFalse(_check2)
        self.assertFalse(_config.valid)
        return

    @mock.patch('argparse.ArgumentParser', new=MockArgumentParser03)
    def test_init_05(self):
        self.assertIsNotNone(bbutil.module)

        _config = AppConfig(use_parser=True)

        _check2 = _config.init()
        self.assertFalse(_check2)
        self.assertFalse(_config.valid)
        return

    def test_init_06(self):
        self.assertIsNotNone(bbutil.module)

        _work = full_path("{0:s}/test".format(os.getcwd()))
        if os.path.exists(_work) is False:
            os.mkdir(_work)

        if sys.platform == "win32":
            _filename = full_path("{0:s}/testdata/config01_win.json".format(os.getcwd()))
        else:
            _filename = full_path("{0:s}/testdata/config01.json".format(os.getcwd()))

        _config = AppConfig(use_config=True, config_filename=_filename)

        _check2 = _config.init()
        self.assertTrue(_check2)
        self.assertTrue(_config.valid)
        self.assertEqual(_config.verbose, 0)
        self.assertEqual(_config.bla, "/usr/local/bin/bla")
        self.assertEqual(_config.bleb, 10)
        self.assertEqual(_config.ls, _ls)
        self.assertEqual(_config.work, _work)
        return

    def test_init_07(self):
        self.assertIsNotNone(bbutil.module)

        _config = AppConfig(use_config=True, config_filename="pufffy")

        _check2 = _config.init()
        self.assertFalse(_check2)
        self.assertFalse(_config.valid)
        return

    def test_init_08(self):
        self.assertIsNotNone(bbutil.module)

        if sys.platform == "win32":
            _filename = full_path("{0:s}/testdata/config02_win.json".format(os.getcwd()))
        else:
            _filename = full_path("{0:s}/testdata/config02.json".format(os.getcwd()))

        _config = AppConfig(use_config=True, config_filename=_filename)

        _check2 = _config.init()
        self.assertFalse(_check2)
        self.assertFalse(_config.valid)
        return

    def test_init_09(self):
        self.assertIsNotNone(bbutil.module)

        _filename = full_path("{0:s}/testdata/config03.json".format(os.getcwd()))
        _config = AppConfig(use_config=True, config_filename=_filename)

        _check2 = _config.init()
        self.assertFalse(_check2)
        self.assertFalse(_config.valid)
        return

    @mock.patch('argparse.ArgumentParser', new=MockArgumentParser04)
    def test_init_10(self):
        self.assertIsNotNone(bbutil.module)

        _config = AppConfig(use_parser=True)

        _check2 = _config.init()
        self.assertFalse(_check2)
        self.assertFalse(_config.valid)
        return

    @mock.patch('argparse.ArgumentParser', new=MockArgumentParser05)
    def test_init_11(self):
        self.assertIsNotNone(bbutil.module)

        _config = AppConfig(use_parser=True)

        _check2 = _config.init()
        self.assertFalse(_check2)
        self.assertFalse(_config.valid)
        return

    def test_init_12(self):
        self.assertIsNotNone(bbutil.module)

        if sys.platform == "win32":
            _filename = full_path("{0:s}/testdata/config04_win.json".format(os.getcwd()))
        else:
            _filename = full_path("{0:s}/testdata/config04.json".format(os.getcwd()))

        _config = AppConfig(use_config=True, config_filename=_filename)

        _check2 = _config.init()
        self.assertFalse(_check2)
        self.assertFalse(_config.valid)
        return

    def test_store_01(self):
        self.assertIsNotNone(bbutil.module)

        _work = full_path("{0:s}/test".format(os.getcwd()))
        if os.path.exists(_work) is False:
            os.mkdir(_work)

        if sys.platform == "win32":
            _filename1 = full_path("{0:s}/testdata/config01_win.json".format(os.getcwd()))
        else:
            _filename1 = full_path("{0:s}/testdata/config01.json".format(os.getcwd()))

        _filename2 = full_path("{0:s}/config01.json".format(os.getcwd()))

        _config = AppConfig(use_config=True, config_filename=_filename1)
        _check2 = _config.init()

        if os.path.exists(_filename2) is True:
            os.remove(_filename2)

        _config.config_filename = _filename2

        _check3 = _config.store()
        _check4 = os.path.exists(_filename2)

        _data = openjson(_filename2)
        if os.path.exists(_filename2) is True:
            os.remove(_filename2)

        self.assertTrue(_check2)
        self.assertTrue(_check3)
        self.assertTrue(_check4)
        self.assertTrue(_config.valid)

        self.assertEqual(_data["verbose"], 0)
        self.assertEqual(_data["bla"], "/usr/local/bin/bla")
        self.assertEqual(_data["bleb"], 10)
        self.assertEqual(_data["ls"], _ls)
        self.assertEqual(_data["work"], _work)
        return

    def test_store_02(self):
        self.assertIsNotNone(bbutil.module)

        _work = full_path("{0:s}/test".format(os.getcwd()))
        if os.path.exists(_work) is False:
            os.mkdir(_work)

        if sys.platform == "win32":
            _filename1 = full_path("{0:s}/testdata/config01_win.json".format(os.getcwd()))
        else:
            _filename1 = full_path("{0:s}/testdata/config01.json".format(os.getcwd()))

        _filename2 = full_path("{0:s}/config01.json".format(os.getcwd()))

        _config = AppConfig(use_config=True, config_filename=_filename1)
        _check2 = _config.init()

        if os.path.exists(_filename2) is True:
            os.remove(_filename2)

        _config.use_config = False

        _check3 = _config.store()
        _check4 = os.path.exists(_filename2)

        self.assertTrue(_check2)
        self.assertTrue(_check3)
        self.assertFalse(_check4)
        self.assertTrue(_config.valid)
        return

    def test_store_03(self):
        self.assertIsNotNone(bbutil.module)

        _work = full_path("{0:s}/test".format(os.getcwd()))
        if os.path.exists(_work) is False:
            os.mkdir(_work)

        if sys.platform == "win32":
            _filename1 = full_path("{0:s}/testdata/config01_win.json".format(os.getcwd()))
        else:
            _filename1 = full_path("{0:s}/testdata/config01.json".format(os.getcwd()))

        _filename2 = full_path("{0:s}/config01.json".format(os.getcwd()))

        _config = AppConfig(use_config=True, config_filename=_filename1)
        _check2 = _config.init()

        if os.path.exists(_filename2) is True:
            os.remove(_filename2)

        _config.config_filename = ""

        _check3 = _config.store()
        _check4 = os.path.exists(_filename2)

        self.assertTrue(_check2)
        self.assertFalse(_check3)
        self.assertFalse(_check4)
        self.assertFalse(_config.valid)
        return

    def test_store_04(self):
        self.assertIsNotNone(bbutil.module)

        _work = full_path("{0:s}/test".format(os.getcwd()))
        if os.path.exists(_work) is False:
            os.mkdir(_work)

        if sys.platform == "win32":
            _filename1 = full_path("{0:s}/testdata/config01_win.json".format(os.getcwd()))
        else:
            _filename1 = full_path("{0:s}/testdata/config01.json".format(os.getcwd()))

        _filename2 = full_path("{0:s}/config01.json".format(os.getcwd()))

        _config = AppConfig(use_config=True, config_filename=_filename1)
        _check2 = _config.init()

        if os.path.exists(_filename2) is True:
            os.remove(_filename2)

        _config.config_filename = _filename2

        with mock.patch('builtins.open', new=mock_open):
            _check3 = _config.store()

        _check4 = os.path.exists(_filename2)

        self.assertTrue(_check2)
        self.assertFalse(_check3)
        self.assertFalse(_check4)
        self.assertTrue(_config.valid)
        return



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#
#    Copyright (C) 2017, Kai Raphahn <kai.raphahn@laburec.de>
#

import os
import unittest
import unittest.mock as mock

from unittest.mock import Mock

import bbutil

from bbutil.database import Table, Types, SQLite

from tests.helper import get_sqlite, set_log, copy_sqlite
from tests.helper.table import TestData, get_table_01, get_table_02, get_table_03, get_table_04

from tests.helper.sqlite import get_sqlite_operational_error, get_sqlite_integrity_error, get_sqlite_return_false

__all__ = [
    "TestTable"
]

def return_count_01() -> int:
    return -1


class TestTable(unittest.TestCase):
    """Testing class for locking module."""

    def setUp(self):
        set_log()
        return

    @staticmethod
    def _clean(sqlite: SQLite):
        _con = None

        if sqlite.manager is not None:
            _con = sqlite.manager.connection

        if _con is not None:
            _con.close()

        if os.path.exists(sqlite.filename) is True:
            os.remove(sqlite.filename)
        return

    def assertHasAttr(self, obj, intended_attr):
        _testBool = hasattr(obj, intended_attr)

        self.assertTrue(_testBool, msg=f'obj lacking an attribute. {obj=}, {intended_attr=}')
        return

    def test_add_column_01(self):

        _sqlite = get_sqlite(filename="test.sqlite", clean=True)

        _table = Table(name="test01", sqlite=_sqlite)
        _table.add_column(name="testid", data_type=Types.integer, unique=True, keyword=True)
        _table.add_column(name="use_test", data_type=Types.bool)
        _table.add_column(name="testname", data_type=Types.string)
        _table.add_column(name="path", data_type=Types.string)

        _names = [
            "testid",
            "use_test",
            "testname",
            "path"
        ]

        _column_list = [
            '"testid" INTEGER',
            '"use_test" BOOLEAN',
            '"testname" TEXT',
            '"path" TEXT'
        ]

        _unique_list = [
            "testid"
        ]

        _count1 = len(_table.names)
        _count2 = len(_table.columns)

        self.assertEqual(_table.keyword, "testid")
        self.assertEqual(_count1, 4)
        self.assertEqual(_count2, 4)
        self.assertListEqual(_names, _table.names)
        self.assertListEqual(_column_list, _table.column_list)
        self.assertListEqual(_unique_list, _table.unique_list)
        self._clean(_table.sqlite)
        return

    def test_add_column_02(self):

        _sqlite = get_sqlite(filename="test.sqlite", clean=True)

        _table = Table(name="test01", sqlite=_sqlite)
        _table.add_column(name="testid", data_type=Types.integer, unique=True, keyword=True)
        _table.add_column(name="testid", data_type=Types.integer)

        _count1 = len(_table.names)

        self.assertEqual(_count1, 1)
        self._clean(_table.sqlite)
        return

    def test_new_data_01(self):

        _sqlite = get_sqlite(filename="test.sqlite", clean=True)
        _table = get_table_01(sqlite_object=_sqlite)

        _data = _table.new_data()
        self.assertHasAttr(_data, "testid")
        self.assertHasAttr(_data, "use_test")
        self.assertHasAttr(_data, "testname")
        self.assertHasAttr(_data, "path")
        self._clean(_table.sqlite)
        return

    def test_init_01(self):
        _sqlite = get_sqlite(filename="test_check_table.sqlite", path="testdata/database")
        _table = get_table_01(sqlite_object=_sqlite)

        _check = _table.init()
        self.assertTrue(_check)
        return

    def test_init_02(self):
        _sqlite = get_sqlite(filename="test_check_table.sqlite", path="testdata/database")
        _table = Table(name="test01", sqlite=_sqlite)

        _check = _table.init()
        self.assertFalse(_check)
        return

    @mock.patch('sqlite3.connect', new=get_sqlite_operational_error())
    def test_init_03(self):
        _sqlite = get_sqlite(filename="test_check_table.sqlite", path="testdata/database")
        _table = get_table_01(sqlite_object=_sqlite)

        _check = _table.init()
        self.assertFalse(_check)
        return

    def test_init_04(self):
        bbutil.set_log(None)

        _table = Table(name="Testos")
        _check1 = _table.init()
        self.assertFalse(_check1)
        return

    def test_init_05(self):
        _sqlite = copy_sqlite(filename="test_check_table.sqlite", path="testdata/database")
        _table = get_table_01(sqlite_object=_sqlite)

        _table.old_name = "tester01"
        _table.name = "tester_new"

        _check = _table.init()
        self.assertTrue(_check)

        self._clean(_sqlite)
        return

    @mock.patch('bbutil.database.sqlite.SQLite.count', new=Mock(return_value=-1))
    def test_init_06(self):
        _sqlite = copy_sqlite(filename="test_check_table.sqlite", path="testdata/database")
        _table = get_table_01(sqlite_object=_sqlite)

        _table.old_name = "tester01"
        _table.name = "tester_new"

        _check = _table.init()
        self.assertFalse(_check)

        self._clean(_sqlite)
        return

    @mock.patch('bbutil.database.sqlite.SQLite.rename_table', new=Mock(return_value=False))
    def test_init_07(self):
        _sqlite = copy_sqlite(filename="test_check_table.sqlite", path="testdata/database")
        _table = get_table_01(sqlite_object=_sqlite)

        _table.old_name = "tester01"
        _table.name = "tester_new"

        _check = _table.init()
        self.assertFalse(_check)

        self._clean(_sqlite)
        return

    def test_upgrade_01(self):
        _sqlite = copy_sqlite(filename="test_database.sqlite", path="testdata/database")

        _table = Table(name="tester01", sqlite=_sqlite)
        _table.add_column(name="testid", data_type=Types.integer, primarykey=True)
        _table.add_column(name="use_test", data_type=Types.bool)
        _table.add_column(name="testname", data_type=Types.string)
        _table.add_column(name="path", data_type=Types.string)
        _table.add_column(name="use_test_name", data_type=Types.string)

        _check = _table.init()
        self.assertTrue(_check)

        _check = _table.check_scheme()
        self.assertFalse(_check)

        _check = _table.upgrade()
        self.assertTrue(_check)

        _check = _table.check_scheme()
        self.assertTrue(_check)

        self._clean(_sqlite)
        return

    def test_upgrade_02(self):
        _sqlite = copy_sqlite(filename="test_database.sqlite", path="testdata/database")

        _table = Table(name="tester01", sqlite=_sqlite)
        _table.add_column(name="testid", data_type=Types.integer, primarykey=True)
        _table.add_column(name="use_test", data_type=Types.bool)
        _table.add_column(name="testname", data_type=Types.string)
        _table.add_column(name="path", data_type=Types.string)

        _check = _table.init()
        self.assertTrue(_check)

        _check = _table.upgrade()
        self.assertTrue(_check)

        self._clean(_sqlite)
        return

    def test_upgrade_03(self):
        _sqlite = copy_sqlite(filename="test_database.sqlite", path="testdata/database")

        _table = Table(name="tester01", sqlite=_sqlite)
        _table.add_column(name="testid", data_type=Types.integer, primarykey=True)
        _table.add_column(name="use_test", data_type=Types.string)
        _table.add_column(name="testname", data_type=Types.string)
        _table.add_column(name="path", data_type=Types.string)

        _check = _table.init()
        self.assertTrue(_check)

        _check = _table.upgrade()
        self.assertFalse(_check)

        self._clean(_sqlite)
        return

    def test_upgrade_04(self):
        _sqlite = copy_sqlite(filename="test_database.sqlite", path="testdata/database")

        _table = Table(name="tester01", sqlite=_sqlite)
        _table.add_column(name="testid", data_type=Types.integer, primarykey=True)
        _table.add_column(name="use_test", data_type=Types.bool)
        _table.add_column(name="testname", data_type=Types.string)
        _table.add_column(name="path", data_type=Types.string)
        _table.add_column(name="use_test_id", data_type=Types.integer, primarykey=True)

        _check = _table.init()
        self.assertTrue(_check)

        _check = _table.upgrade()
        self.assertFalse(_check)

        self._clean(_sqlite)
        return

    def test_upgrade_05(self):
        _sqlite = copy_sqlite(filename="test_database.sqlite", path="testdata/database")

        _table = Table(name="tester01", sqlite=_sqlite)
        _table.add_column(name="testid", data_type=Types.integer, primarykey=True)
        _table.add_column(name="use_test", data_type=Types.bool)
        _table.add_column(name="testname", data_type=Types.string)
        _table.add_column(name="path", data_type=Types.string)
        _table.add_column(name="use_test_id", data_type=Types.integer, unique=True)

        _check = _table.init()
        self.assertTrue(_check)

        _check = _table.upgrade()
        self.assertFalse(_check)

        self._clean(_sqlite)
        return

    @mock.patch('bbutil.database.sqlite.SQLite.add_columns', new=Mock(return_value=False))
    def test_upgrade_06(self):
        _sqlite = copy_sqlite(filename="test_database.sqlite", path="testdata/database")

        _table = Table(name="tester01", sqlite=_sqlite)
        _table.add_column(name="testid", data_type=Types.integer, primarykey=True)
        _table.add_column(name="use_test", data_type=Types.bool)
        _table.add_column(name="testname", data_type=Types.string)
        _table.add_column(name="path", data_type=Types.string)
        _table.add_column(name="use_test_id", data_type=Types.integer)

        _check = _table.init()
        self.assertTrue(_check)

        _check = _table.upgrade()
        self.assertFalse(_check)

        self._clean(_sqlite)
        return

    def test_upgrade_07(self):
        _sqlite = copy_sqlite(filename="test_database.sqlite", path="testdata/database")

        _table = Table(name="tester01", sqlite=_sqlite)
        _table.add_column(name="testid", data_type=Types.integer, primarykey=True)
        _table.add_column(name="use_test", data_type=Types.bool)
        _table.add_column(name="testname", data_type=Types.string)

        _check = _table.init()
        self.assertTrue(_check)

        _check = _table.check_scheme()
        self.assertFalse(_check)

        _check = _table.upgrade()
        self.assertTrue(_check)

        self._clean(_sqlite)
        return

    @mock.patch('bbutil.database.sqlite.SQLite.drop_columns', new=Mock(return_value=False))
    @mock.patch('bbutil.database.sqlite.SQLite.check_minmal_version', new=Mock(return_value=True))
    def test_upgrade_08(self):
        _sqlite = copy_sqlite(filename="test_database.sqlite", path="testdata/database")

        _table = Table(name="tester01", sqlite=_sqlite)
        _table.add_column(name="testid", data_type=Types.integer, primarykey=True)
        _table.add_column(name="use_test", data_type=Types.bool)
        _table.add_column(name="testname", data_type=Types.string)

        _check = _table.init()
        self.assertTrue(_check)

        _check = _table.check_scheme()
        self.assertFalse(_check)

        _check = _table.upgrade()
        self.assertFalse(_check)

        self._clean(_sqlite)
        return

    @mock.patch('bbutil.database.sqlite.SQLite.check_minmal_version', new=Mock(return_value=False))
    def test_upgrade_09(self):
        _sqlite = copy_sqlite(filename="test_database.sqlite", path="testdata/database")

        _table = Table(name="tester01", sqlite=_sqlite)
        _table.add_column(name="testid", data_type=Types.integer, primarykey=True)
        _table.add_column(name="use_test", data_type=Types.bool)
        _table.add_column(name="testname", data_type=Types.string)

        _check = _table.init()
        self.assertTrue(_check)

        _check = _table.check_scheme()
        self.assertFalse(_check)

        _check = _table.upgrade()
        self.assertTrue(_check)

        _check = _table.check_scheme()
        self.assertFalse(_check)

        self._clean(_sqlite)
        return

    def test_get_column_01(self):
        _table = Table(name="tester01")
        _table.add_column(name="testid", data_type=Types.integer, primarykey=True)
        _table.add_column(name="use_test", data_type=Types.bool)
        _table.add_column(name="testname", data_type=Types.string)
        _table.add_column(name="path", data_type=Types.string)
        _table.add_column(name="use_test_id", data_type=Types.integer)

        _column = _table.get_column("testid")
        self.assertIsNotNone(_column)

        self.assertEqual(_column.name, "testid")
        self.assertTrue(_column.primarykey)
        self.assertEqual(_column.type, Types.integer)

        _column = _table.get_column("testidxx")
        self.assertIsNone(_column)
        return

    def test_check_scheme_01(self):
        _sqlite = get_sqlite(filename="test_database.sqlite", path="testdata/database")

        _table = Table(name="tester01", sqlite=_sqlite)
        _table.add_column(name="testid", data_type=Types.integer, primarykey=True)
        _table.add_column(name="use_test", data_type=Types.bool)
        _table.add_column(name="testname", data_type=Types.string)
        _table.add_column(name="path", data_type=Types.string)

        _check1 = _table.init()
        _check2 = _table.check_scheme()

        self.assertTrue(_check1)
        self.assertTrue(_check2)
        return

    def test_check_scheme_02(self):
        _sqlite = get_sqlite(filename="test_database.sqlite", path="testdata/database")

        _table = Table(name="tester01", sqlite=_sqlite)
        _table.add_column(name="testid", data_type=Types.integer, primarykey=True)
        _table.add_column(name="use_test", data_type=Types.string)
        _table.add_column(name="testname", data_type=Types.string)
        _table.add_column(name="path", data_type=Types.string)

        _check1 = _table.init()
        _check2 = _table.check_scheme()
        _count1 = len(_table.missing_columns)
        _count2 = len(_table.invalid_columns)

        self.assertEqual(_count1, 0)
        self.assertEqual(_count2, 1)
        self.assertTrue(_check1)
        self.assertFalse(_check2)
        return

    def test_check_scheme_03(self):
        _sqlite = get_sqlite(filename="test_database.sqlite", path="testdata/database")

        _table = Table(name="tester01", sqlite=_sqlite)
        _table.add_column(name="testid", data_type=Types.integer, primarykey=True)
        _table.add_column(name="use_test_now", data_type=Types.bool)
        _table.add_column(name="testname", data_type=Types.string)
        _table.add_column(name="path", data_type=Types.string)

        _check1 = _table.init()
        _check2 = _table.check_scheme()
        _count1 = len(_table.missing_columns)
        _count2 = len(_table.invalid_columns)

        self.assertEqual(_count1, 1)
        self.assertEqual(_count2, 0)
        self.assertTrue(_check1)
        self.assertFalse(_check2)
        return

    def test_check_scheme_04(self):
        _sqlite = get_sqlite(filename="test_database.sqlite", path="testdata/database")

        _table = Table(name="tester04", sqlite=_sqlite)
        _table.add_column(name="testid", data_type=Types.integer, primarykey=True)
        _table.add_column(name="use_test", data_type=Types.bool)
        _table.add_column(name="testname", data_type=Types.string)
        _table.add_column(name="path", data_type=Types.string)

        _table.sqlite.prepare()
        _check1 = _table.check_scheme()

        self.assertFalse(_check1)
        return

    def test_check_scheme_05(self):
        _sqlite = get_sqlite(filename="test_database.sqlite", path="testdata/database")

        _table = Table(name="tester04", sqlite=_sqlite)

        _table.sqlite.prepare()
        _check1 = _table.check_scheme()

        self.assertFalse(_check1)
        return

    def test_check_scheme_06(self):
        bbutil.set_log(None)
        _sqlite = get_sqlite(filename="test_database.sqlite", path="testdata/database")

        _table = Table(name="tester04", sqlite=_sqlite)

        _table.sqlite.prepare()
        _check1 = _table.check_scheme()

        self.assertFalse(_check1)
        return

    def test_check_scheme_07(self):
        _sqlite = get_sqlite(filename="test_database.sqlite", path="testdata/database")

        _table = Table(name="tester01", sqlite=_sqlite)
        _table.add_column(name="testid", data_type=Types.integer, primarykey=True)
        _table.add_column(name="use_test", data_type=Types.bool)
        _table.add_column(name="testname", data_type=Types.string)

        _table.sqlite.prepare()
        _check1 = _table.check_scheme()

        self.assertFalse(_check1)
        return

    def test_select_01(self):
        _sqlite = get_sqlite(filename="test_select.sqlite", path="testdata/database")
        _table = get_table_01(sqlite_object=_sqlite)

        _check2 = _table.init()

        _data = _table.select()
        _count = len(_data)

        self.assertTrue(_check2)
        self.assertEqual(_count, 6)
        return

    def test_select_02(self):
        _sqlite = get_sqlite(filename="test_check_table.sqlite", path="testdata/database")
        _table = get_table_01(sqlite_object=_sqlite)
        _table.suppress_warnings = False

        _check2 = _table.init()

        _data = _table.select()
        _count = len(_data)

        self.assertTrue(_check2)
        self.assertEqual(_count, 0)
        return

    def test_select_03(self):
        _sqlite = get_sqlite(filename="test_select.sqlite", path="testdata/database")
        _table = get_table_01(sqlite_object=_sqlite)
        _table.suppress_warnings = False

        _check2 = _table.init()

        with mock.patch('sqlite3.connect', new=get_sqlite_operational_error()):
            _data = _table.select()

        _count = len(_data)

        self.assertTrue(_check2)
        self.assertEqual(_count, 0)
        return

    def test_select_04(self):
        _sqlite = get_sqlite(filename="test_select.sqlite", path="testdata/database")
        _table = get_table_02(sqlite_object=_sqlite)

        _check2 = _table.init()

        _data = _table.select()
        _count = len(_data)

        self.assertTrue(_check2)
        self.assertEqual(_count, 0)
        return

    def test_store_01(self):
        _sqlite = get_sqlite(filename="test.sqlite", clean=True)
        _table = get_table_01(sqlite_object=_sqlite)

        _check2 = _table.init()

        _data = _table.new_data()
        _data.use_test = True
        _data.testname = "Test01"
        _data.path = "path"

        _count = _table.store(_data)

        self.assertTrue(_check2)
        self.assertEqual(_count, 1)
        self._clean(_table.sqlite)
        return

    def test_store_02(self):
        _sqlite = get_sqlite(filename="test.sqlite", clean=True)
        _table = get_table_01(sqlite_object=_sqlite)

        _check2 = _table.init()

        _data1 = _table.new_data()
        _data1.testid = 0
        _data1.use_test = True
        _data1.testname = "Test01"
        _data1.path = "path"

        _data2 = _table.new_data()
        _data2.testid = 1
        _data2.use_test = True
        _data2.testname = "Test02"
        _data2.path = "path"

        _table.add(_data1)
        _table.add(_data2)

        _count = _table.store()

        self.assertTrue(_check2)
        self.assertEqual(_count, 2)
        self._clean(_table.sqlite)
        return

    def test_store_03(self):
        _sqlite = get_sqlite(filename="test.sqlite", clean=True)
        _table = get_table_01(sqlite_object=_sqlite)

        _check2 = _table.init()

        _data1 = _table.new_data()
        _data1.testid = 0
        _data1.use_test = True
        _data1.testname = "Test01"
        _data1.path = "path"

        _data2 = _table.new_data()
        _data2.testid = 1
        _data2.use_test = True
        _data2.testname = "Test02"
        _data2.path = "path"

        _data3 = _table.new_data()
        _data3.testid = 1
        _data3.use_test = True
        _data3.testname = "Test02"
        _data3.path = "path"

        _table.add(_data1)
        _table.add(_data2)
        _table.add(_data3)

        _count = _table.store()

        self.assertTrue(_check2)
        self.assertEqual(_count, 2)
        self._clean(_table.sqlite)
        return

    def test_add_01(self):
        _sqlite = get_sqlite(filename="test.sqlite", clean=True)
        _table = get_table_04(sqlite_object=_sqlite)

        _check2 = _table.init()

        _data1 = _table.new_data()
        _data1.use_test = True
        _data1.category = "TestMain"
        _data1.testname = "Test01"
        _data1.path = "path"

        _data2 = _table.new_data()
        _data2.use_test = True
        _data2.category = "TestMain"
        _data2.testname = "Test02"
        _data2.path = "path"

        _data3 = _table.new_data()
        _data3.use_test = True
        _data3.category = "TestOther"
        _data3.testname = "Test02"
        _data3.path = "path"

        _table.add(_data1)
        _table.add(_data2)
        _table.add(_data3)

        _list1 = _table.index["TestMain"]
        _list2 = _table.index["TestOther"]

        self.assertTrue(_check2)
        self.assertEqual(len(_list1), 2)
        self.assertEqual(len(_list2), 1)
        self._clean(_table.sqlite)
        return

    def test_add_02(self):
        _sqlite = get_sqlite(filename="test.sqlite", clean=True)
        _table = get_table_04(sqlite_object=_sqlite)

        _check2 = _table.init()

        _data1 = _table.new_data()
        _data1.use_test = True
        _data1.category = "TestMain"
        _data1.testname = "Test01"
        _data1.path = "path"

        _data2 = TestData()
        _data2.use_test = True
        _data2.xcategory = "TestMain"
        _data2.testname = "Test01"
        _data2.path = "path"

        _table.add(_data1)

        # noinspection PyTypeChecker
        self.assertRaises(Exception, _table.add, _data2)

        _list1 = _table.index["TestMain"]

        self.assertTrue(_check2)
        self.assertEqual(len(_list1), 1)
        self._clean(_table.sqlite)
        return

    def test_update_01(self):
        _sqlite = get_sqlite(filename="test.sqlite", clean=True)
        _table = get_table_04(sqlite_object=_sqlite)

        _check2 = _table.init()

        _data1 = _table.new_data()
        _data1.use_test = True
        _data1.category = "TestMain"
        _data1.testname = "Test01"
        _data1.path = "path"

        _table.add(_data1)
        _check3 = _table.store()

        _data2 = _table.new_data()
        _data2.use_test = True
        _data2.category = "TestMain"
        _data2.testname = "Test02"
        _data2.path = "path"

        _check4 = _table.update(_data2, "testid = ?", 1)
        _count = _table.check()

        self.assertTrue(_check2)
        self.assertTrue(_check3)
        self.assertTrue(_check4)
        self.assertEqual(_count, 1)
        self._clean(_table.sqlite)
        return

    def _load_table(self, table: Table, limit: int):
        _sql_count = table.check()
        _data_count = table.load()

        self.assertEqual(_sql_count, limit)
        self.assertEqual(_data_count, limit)
        return

    def test_load_01(self):
        _sqlite = get_sqlite(filename="test_bulk.sqlite", path="testdata/database")

        _table0 = get_table_03("interval0", sqlite_object=_sqlite)
        _table1 = get_table_03("interval1", sqlite_object=_sqlite)
        _table2 = get_table_03("interval2", sqlite_object=_sqlite)
        _table3 = get_table_03("interval3", sqlite_object=_sqlite)
        _table4 = get_table_03("interval4", sqlite_object=_sqlite)
        _table5 = get_table_03("interval5", sqlite_object=_sqlite)
        _table6 = get_table_03("interval6", sqlite_object=_sqlite)

        _tables = [
            _table0,
            _table1,
            _table2,
            _table3,
            _table4,
            _table5,
            _table6
        ]

        for _table in _tables:
            _check = _table.init()
            self.assertTrue(_check)

        self._load_table(_table0, 500)
        self._load_table(_table1, 1000)
        self._load_table(_table2, 5000)
        self._load_table(_table3, 10000)
        self._load_table(_table4, 20000)
        self._load_table(_table5, 50000)
        self._load_table(_table6, 100000)
        return

    def test_load_02(self):
        _sqlite = get_sqlite(filename="test_select.sqlite", path="testdata/database")

        _table = get_table_01(sqlite_object=_sqlite)

        _check_init = _table.init()

        with mock.patch('sqlite3.connect', new=get_sqlite_operational_error()):
            _count = _table.load()

        self.assertTrue(_check_init)
        self.assertEqual(_count, 0)
        return

    def test_clear_01(self):
        _sqlite = get_sqlite(filename="test_select.sqlite", path="testdata/database")

        _table = get_table_01(sqlite_object=_sqlite)

        _check_init = _table.init()
        _count1 = _table.load()

        _table.clear()

        _count2 = _table.data_count

        self.assertTrue(_check_init)
        self.assertEqual(_count1, 6)
        self.assertEqual(_count2, 0)
        return



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#
#    Copyright (C) 2017, Kai Raphahn <kai.raphahn@laburec.de>
#

import os.path

from typing import Optional, List, TextIO
from enum import Enum
from argparse import ArgumentParser

from bbutil.lang.parser.domain import Domain
from bbutil.lang.parser.language import Languages

from bbutil.utils import full_path
from bbutil.lang.parser.pyfile import PythonFile
from bbutil.logging import Logging

__all__ = [
    "pyfile",
    "domain",
    "language",

    "Command",
    "Parser"
]

log: Optional[Logging] = None


def set_log(logging: Logging):
    global log
    log = logging
    return


class Command(Enum):

    generate = "Generate"
    update = "Update"
    copy = "Copy"
    compile = "Compile"


def sort_files(python_file: PythonFile):
    return python_file.basename


class Parser(object):

    def __init__(self):
        self._argument_parser: ArgumentParser = ArgumentParser()

        self._ext: str = ""
        self._echo: str = '"'
        self.log: Optional[Logging] = None

        self.is_windows: bool = False
        self.root_path: str = os.getcwd()
        self.package_path: str = os.getcwd()
        self.module: str = ""
        self.module_filter: str = ""

        self.script: List[str] = []
        self.locales: str = ""

        self._python_files: List[PythonFile] = []
        self._domains: List[Domain] = []

        os.environ["IGNORE_GETTEXT"] = "True"
        return

    @property
    def file_number(self) -> int:
        return len(self._python_files)

    def init(self, args: list) -> bool:
        _argument_parser = ArgumentParser()

        _argument_parser.add_argument("-r", "--rootpath", help="root path", type=str, default=os.getcwd())
        _argument_parser.add_argument("-p", "--packagepath", help="package path", type=str, default=os.getcwd())
        _argument_parser.add_argument("-l", "--locales", help="locales output path", type=str, default="locale")

        _argument_parser.add_argument("-m", "--module", help="module name of package", type=str)
        _argument_parser.add_argument("-f", "--filter", help="filter for module names", type=str)
        _argument_parser.add_argument("-w", "--windows", help="used on windows", action='store_true')

        _argument_parser.add_argument("-s", "--script", help="script names seperated by comma", type=str)
        _argument_parser.add_argument("-v", "--verbose", help="increase output verbosity", type=int,
                                      choices=[0, 1, 2, 3], default=0)

        options = _argument_parser.parse_args(args)

        _scripts = []

        if options.script is not None:
            _items = options.script.split(",")
            for _item in _items:
                _scripts.append(_item)

        check = self.setup(root_path=options.rootpath,
                           package_path=options.packagepath,
                           locales=options.locales,
                           module=options.module,
                           filter=options.filter,
                           script=_scripts,
                           windows=options.windows)

        if check is False:
            return False

        if options.verbose is not None:
            log.setup(level=options.verbose)

        self.log = log
        return True

    def setup(self, **kwargs) -> bool:
        item = kwargs.get("root_path", None)
        if item is not None:
            self.root_path = item

        item = kwargs.get("package_path", None)
        if item is not None:
            self.package_path = item

        item = kwargs.get("locales", None)
        if item is not None:
            self.locales = item

        item = kwargs.get("module", None)
        if item is not None:
            self.module = item

        item = kwargs.get("filter", None)
        if item is not None:
            self.module_filter = item

        item = kwargs.get("windows", None)
        if item is not None:
            if item is True:
                self.is_windows = item
                self._ext = ".exe"
                self._echo = ""

        item = kwargs.get("script", None)
        if item is not None:
            self.script = item

        if (self.module == "") or (self.locales == ""):
            log.error("Need module name or locales path!")
            return False

        return True

    def get_domain(self, domain_name: str) -> Optional[Domain]:

        for _domain in self._domains:
            if _domain.domain == domain_name:
                return _domain

        return None

    def _parse_script(self, filename: str) -> bool:
        _package_path = os.path.dirname(filename)

        log.debug2("Parse", filename)
        _file = PythonFile(package_path=_package_path,
                           filename=filename,
                           module="")

        check = _file.create()
        if check is True:
            self._python_files.append(_file)
        return check

    def _parse_package(self) -> bool:
        file_list = []

        log.inform("Check", self.package_path)
        for root, dirs, files in os.walk(self.package_path, topdown=True):
            for name in files:
                _filename = full_path("{0:s}/{1:s}".format(root, name))
                if "__pycache__" in _filename:
                    continue
                file_list.append(_filename)

        for _filename in sorted(file_list):
            log.debug2("Parse", _filename)
            _info = "Module: {0:s}, Filter: {1:s}".format(self.module, self.module_filter)
            log.debug2("Debug", _info)
            _file = PythonFile(package_path=self.package_path,
                               filename=_filename,
                               module=self.module,
                               module_filter=self.module_filter)
            check = _file.create()
            if check is False:
                continue

            self._python_files.append(_file)
        return True

    def parse(self) -> bool:
        os.environ["IGNORE_GETTEXT"] = "True"

        if self.script != "":
            for _item in sorted(self.script):
                _file = full_path(_item)

                _check = self._parse_script(_file)
                if _check is False:
                    continue

        self._parse_package()

        for _file in self._python_files:
            _domain = self.get_domain(_file.domain)
            if _domain is None:
                _domain = Domain(root_path=self.root_path, domain=_file.domain)
                _domain.files.append(_file)
                _domain.lang.append(Languages(self.locales, 'en', _domain.domain))
                _domain.lang.append(Languages(self.locales, 'de', _domain.domain))
                self._domains.append(_domain)
            else:
                _domain.files.append(_file)

        log.inform("Files", str(len(self._python_files)))
        log.inform("Domain", str(len(self._domains)))
        return True

    def generate(self):
        script_line = []
        for _file in self._python_files:
            _pot = os.path.basename(_file.pot)

            _comment = 'echo {1:s}Create {0:s}{1:s}'.format(_pot, self._echo)
            _command = "xgettext{0:s} -L python -d {1:s} -o {2:s} {3:s}".format(self._ext,
                                                                                _file.domain,
                                                                                _file.pot,
                                                                                _file.fullname)

            script_line.append(_comment)
            script_line.append(_command)

        for _domain in self._domains:
            _pot = os.path.basename(_domain.pot)

            _comment = 'echo {1:s}Merge {0:s}{1:s}'.format(_pot, self._echo)
            _command = "msgcat{0:s}".format(self._ext)

            for _file in _domain.files:
                _command = "{0:s} {1:s}".format(_command, _file.pot)

            _command = "{0:s} -o {1:s}".format(_command, _domain.pot)

            script_line.append(_comment)
            script_line.append(_command)
        return script_line

    def copy(self):
        script_line = []
        for _domain in self._domains:
            for _lang in _domain.lang:
                _comment = 'echo {2:s}Update {0:s}/{1:s}{2:s}'.format(_lang.lang, _domain.domain, self._echo)

                if self.is_windows is True:
                    _command = "copy {0:s} {1:s}".format(_domain.pot, _lang.po)
                else:
                    _command = "cp {0:s} {1:s}".format(_domain.pot, _lang.po)

                script_line.append(_comment)
                script_line.append(_command)
        return script_line

    def update(self) -> list:
        script_line = []
        for _domain in self._domains:
            for _lang in _domain.lang:
                _comment = 'echo {2:s}Update {0:s}/{1:s}{2:s}'.format(_lang.lang, _domain.domain, self._echo)
                _command = "msgmerge{0:s} -N -U {1:s} {2:s}".format(self._ext, _lang.po, _domain.pot)

                script_line.append(_comment)
                script_line.append(_command)
        return script_line

    def compile(self) -> list:
        _root = os.getcwd()
        script_line = []

        for _domain in self._domains:
            for _lang in _domain.lang:
                _comment = 'echo {2:s}Compile {0:s}/{1:s}{2:s}'.format(_lang.lang, _domain.domain, self._echo)
                _command = "msgfmt{0:s} -o {1:s} {2:s}".format(self._ext, _lang.mo, _lang.po)
                script_line.append(_comment)
                script_line.append(_command)
        return script_line

    def _open(self, filename: str) -> Optional[TextIO]:
        if self.is_windows is True:
            script_ext = ".cmd"
            first_line = "@echo off\n"
        else:
            script_ext = ".sh"
            first_line = "#!/bin/bash\n"

        _filename = full_path("{0:s}{1:s}".format(filename, script_ext))

        try:
            f = open(_filename, "w")
        except IOError as e:
            log.exception(e)
            return None

        f.write(first_line)

        log.inform("Open", _filename)
        return f

    @staticmethod
    def _write(f: TextIO, lines: List[str]):
        for _item in lines:
            data = "{0:s}\n".format(_item)
            f.write(data)
        return

    def store(self, command: Command) -> bool:
        _name = ""

        if command is Command.generate:
            _name = "_lang-generate"

        if command is Command.update:
            _name = "_lang-update"

        if command is Command.copy:
            _name = "_lang-copy"

        if command is Command.compile:
            _name = "_lang-compile"

        f = self._open(_name)
        if f is None:
            return False

        if command is Command.generate:
            self._write(f, self.generate())

        if command is Command.update:
            self._write(f, self.update())

        if command is Command.copy:
            self._write(f, self.copy())

        if command is Command.compile:
            self._write(f, self.compile())

        f.close()
        return True



#!/usr/bin/python3
# coding=utf-8

# Copyright (C) 2020, Siemens Healthcare Diagnostics Products GmbH
# Licensed under the Siemens Inner Source License 1.2, see LICENSE.md.

import abc
import os.path
import argparse

import json

from dataclasses import dataclass
from abc import ABCMeta
from typing import Optional

from bbutil.utils import check_object, check_dict, openjson

import bbutil

__all__ = [
    "Config"
]


@dataclass
class Config(metaclass=ABCMeta):

    _is_valid: bool = False

    config_filename: str = ""
    use_parser: bool = False
    use_config: bool = False
    parser: Optional[argparse.ArgumentParser] = None
    verbose: int = 0

    @property
    def valid(self) -> bool:
        return self._is_valid

    @staticmethod
    def check_path(input_path) -> bool:
        if input_path == "":
            bbutil.log.error("Invalid path!")
            return False

        if os.path.exists(input_path) is False:
            bbutil.log.error("Unable to find: {0:s}".format(input_path))
            return False
        return True

    def prepare_parser(self) -> bool:
        self.parser: argparse.ArgumentParser = argparse.ArgumentParser()

        if bbutil.module is None:
            bbutil.log.error("Module manager is missing!")
            return False

        commands = []

        for _module in bbutil.module.modules:
            commands.append(_module.command)

        self.parser.add_argument("command", help="commands to run, use <command> -h for more help",
                                 choices=commands)

        self.parser.add_argument("-v", "--verbose", help="increase output verbosity", type=int,
                                 default=0, choices=[0, 1, 2])
        return True

    @abc.abstractmethod
    def setup_parser(self):
        pass

    @abc.abstractmethod
    def read_parser(self, options) -> bool:
        pass

    def _read_parser(self) -> bool:
        options = self.parser.parse_args()

        check = check_object(options, ["verbose"])
        if check is False:
            bbutil.log.error("Parser results are invalid!")
            return False

        self.verbose = options.verbose

        check = self.read_parser(options)
        if check is False:
            bbutil.log.error("Reading config from parser has failed!")
            return False
        return True

    def _init_parser(self) -> bool:
        if self.use_parser is False:
            return True

        check = self.prepare_parser()
        if check is False:
            return False

        self.setup_parser()

        check = self._read_parser()
        if check is False:
            return False

        return True

    @abc.abstractmethod
    def parse_config(self, config: dict) -> bool:
        pass

    def _load_config(self) -> bool:
        if self.use_config is False:
            return True

        if os.path.exists(self.config_filename) is False:
            bbutil.log.error("Config not found: {0:s}".format(self.config_filename))
            return False

        _config = openjson(self.config_filename)

        check = check_dict(_config, ["verbose"])
        if check is False:
            bbutil.log.error("Config is invalid!")
            return False

        check = self.parse_config(_config)
        if check is False:
            bbutil.log.error("Reading config from file has failed!")
            return False
        return True

    @abc.abstractmethod
    def create_config(self) -> dict:
        pass

    def store(self) -> bool:
        if self.use_config is False:
            return True

        if self.config_filename == "":
            self._is_valid = False
            bbutil.log.error("No filename for storing!")
            return False

        _config = self.create_config()
        _data = json.dumps(_config, indent=4)

        bbutil.log.inform("Config", "Store {0:s}".format(self.config_filename))

        try:
            f = open(file=self.config_filename, mode="w")
        except OSError as e:
            bbutil.log.exception(e)
            return False

        f.write(_data)
        f.close()
        return True

    @abc.abstractmethod
    def check_config(self) -> bool:
        pass

    def init(self) -> bool:
        if (self.use_config is False) and (self.use_parser is False):
            self._is_valid = True
            return True

        check = self._init_parser()
        if check is False:
            return False

        check = self._load_config()
        if check is False:
            return False

        check = self.check_config()
        if check is False:
            bbutil.log.error("Config check has failed!")
            return False

        self._is_valid = True
        return True



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#
#    Copyright (C) 2017, Kai Raphahn <kai.raphahn@laburec.de>
#

from typing import List, Optional

from tkinter import Text
from tkinter.constants import END
from tkinter.ttk import Progressbar

from bbutil.logging.types import Writer, Message
from bbutil.utils import get_terminal_size

__all__ = [
    "TkWriter"
]

classname = "TkWriter"
_index = ["INFORM", "DEBUG1", "DEBUG2", "DEBUG3", "WARN", "ERROR", "EXCEPTION", "TIMER", "PROGRESS"]

_schemes = {
    "NORMAL": "",
    "INFORM": "green",
    "DEBUG1": "cyan",
    "DEBUG2": "cyan",
    "DEBUG3": "cyan",
    "WARN": "magenta",
    "ERROR": "red",
    "EXCEPTION": "red",
    "TIMER": "yellow"
}


class TkWriter(Writer):

    def __init__(self):
        Writer.__init__(self, "TkLogger", _index)

        self.text_control: Optional[Text] = None
        self.progress_control: Optional[Progressbar] = None

        self.encoding: str = ""
        self.text_space: int = 15
        self.text_size: int = 11
        self.seperator: str = "|"
        self.length: int = 0
        self.error_index: List[str] = ["ERROR", "EXCEPTION"]

        size_x, size_y = get_terminal_size()

        self.line_width: int = size_x
        self.bar_len: int = 50
        self.active: bool = False
        return

    def setup(self, **kwargs):
        item = kwargs.get("text_space", None)
        if item is not None:
            self.text_space = item

        item = kwargs.get("seperator", None)
        if item is not None:
            self.seperator = item

        item = kwargs.get("error_index", None)
        if item is not None:
            self.error_index = item

        item = kwargs.get("bar_len", None)
        if item is not None:
            self.bar_len = item

        item = kwargs.get("text_size", None)
        if item is not None:
            self.text_size = item

        item = kwargs.get("text_control", None)
        if item is not None:
            self.text_control = item
            self._set_schemmes()

        item = kwargs.get("progress_control", None)
        if item is not None:
            self.progress_control = item
        return

    def _set_schemmes(self):
        for _key in _schemes:
            _value = _schemes[_key]
            if _value == "":
                self.text_control.tag_config(_key, font=("Monospace", self.text_size))
            else:
                self.text_control.tag_config(_key, font=("Monospace", self.text_size), foreground=_value)
        return

    def open(self) -> bool:
        if self.text_control is None:
            return False

        if self.progress_control is None:
            return False

        self.active = True
        return True

    def close(self) -> bool:  # pragma: no cover
        self.active = False
        return True

    def clear(self) -> bool:
        self.progress_control.configure(value=0)
        return True

    def _set_progress(self, item: Message):
        value = int(float(item.progress.counter * 100) / float(item.progress.limit))
        self.progress_control.configure(value=value)
        return

    def write(self, item: Message):
        if item.level == "PROGRESS":
            self._set_progress(item)
            return

        if self.active is False:
            return

        if item.tag == "":
            text = "{0:s}\n".format(item.content)
            self.text_control.insert(END, text, item.level)
            self.text_control.see(END)
        else:
            tag = item.tag.ljust(self.text_space)
            text = "{0:s} {1:s}\n".format(self.seperator, item.content)

            self.text_control.insert(END, tag, item.level)
            self.text_control.insert(END, text, "NORMAL")
            self.text_control.see(END)
        return



#!/usr/bin/python3
# coding=utf-8

# Copyright (C) 2020, Siemens Healthcare Diagnostics Products GmbH
# Licensed under the Siemens Inner Source License 1.2, see LICENSE.md.

import subprocess

from typing import Optional, List

import bbutil

__all__ = [
    "Execute"
]


class Execute(object):

    def __init__(self, **kwargs):
        self.name: str = ""
        self.desc: str = ""
        self.commands: List[str] = []
        self.messages: List[str] = []
        self.returncode: int = 0
        self.errors: Optional[List[str]] = None

        self.stdout: Optional[subprocess.PIPE] = subprocess.PIPE
        self.stderr: Optional[subprocess.PIPE] = subprocess.PIPE
        self.stdin: Optional[subprocess.PIPE] = None
        self.call_stdout = None
        self.call_stderr = None

        self.setup(**kwargs)
        return

    def show_command(self):
        line = ""

        for _item in self.commands:
            if line == "":
                line = _item
            else:
                line = "{0:s} {1:s}".format(line, _item)

        bbutil.log.debug1(self.name, line)
        return

    @staticmethod
    def _convert_line(data) -> str:
        try:
            new = data.decode(encoding='utf-8')
        except UnicodeDecodeError:
            return ""
        new = new.replace("\r\n", "")
        new = new.replace("\n", "")
        return new

    def setup(self, **kwargs):
        item = kwargs.get("commands", None)
        if item is not None:
            self.commands = item

        item = kwargs.get("name", None)
        if item is not None:
            self.name = item

        item = kwargs.get("desc", None)
        if item is not None:
            self.desc = item

        item = kwargs.get("stdout", None)
        if item is not None:
            self.stdout = item

        item = kwargs.get("stderr", None)
        if item is not None:
            self.stderr = item

        item = kwargs.get("stdin", None)
        if item is not None:
            self.stdin = item

        item = kwargs.get("call_stdout", None)
        if item is not None:
            self.call_stdout = item

        item = kwargs.get("call_stderr", None)
        if item is not None:
            self.call_stderr = item
        return

    def execute(self) -> bool:
        self.show_command()

        self.messages = []

        if self.stdin is not None:
            p = subprocess.Popen(self.commands, stdout=self.stdout, stderr=self.stderr, stdin=self.stdin)
        else:
            p = subprocess.Popen(self.commands, stdout=self.stdout, stderr=self.stderr)

        #  parse output and wait for end
        while True:
            if p.stdout is not None:
                for line in p.stdout:
                    if line is None:
                        continue
                    data = self._convert_line(line)
                    self.messages.append(data)

                    if self.call_stdout is not None:
                        self.call_stdout(data)

            if p.stderr is not None:
                for line in p.stderr:
                    if line is None:
                        continue
                    data = self._convert_line(line)

                    if self.errors is None:
                        self.errors = []

                    self.errors.append(data)

                    if self.call_stderr is not None:
                        self.call_stderr(data)

            _poll = p.poll()
            if _poll is not None:
                break

        self.returncode = p.returncode

        if p.returncode != 0:
            bbutil.log.clear()
            bbutil.log.error("{0:s} failed!".format(self.desc))
            bbutil.log.error("Process did end with error {0:d}".format(p.returncode))
            if self.errors is not None:
                for _line in self.errors:
                    bbutil.log.error(_line)
            else:
                for _line in self.messages:
                    bbutil.log.error(_line)
            return False

        return True



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#
#    Copyright (C) 2017, Kai Raphahn <kai.raphahn@laburec.de>
#

import sys
import subprocess
import os
import platform
import json
import struct
from typing import Union, Any, Tuple

__all__ = [
    "check_dict",
    "check_object",
    "openjson",
    "fix_path",
    "fix_sep",
    "full_path",
    "get_attribute",
    "get_terminal_size",
    "print_exception",
    "execute"
]


def check_dict(checkdict: dict, keylist: list) -> bool:
    ret = True
    list_keys = list(checkdict)

    for key in keylist:
        if list_keys.count(key) == 0:
            ret = False

    return ret


def check_object(checkobject: Any, keylist: list) -> bool:
    ret = True

    if checkobject is None:
        return False

    for key in keylist:
        if hasattr(checkobject, key) is False:
            ret = False

    return ret


def openjson(filename: str) -> Union[dict, None]:
    f = open(filename, mode='r', encoding="utf-8")
    data = json.load(f)
    f.close()
    return data


def fix_path(pathname: str) -> str:
    _path = pathname
    if sys.platform == "win32":
        _path = "{0:s}\\".format(pathname)

    return _path


def fix_sep(pathname: str) -> str:
    _path = pathname.replace(os.sep, "/")
    return _path


def full_path(pathname: str) -> str:
    filename = os.path.abspath(os.path.normpath(pathname))
    return filename


def get_attribute(path: str, classname: str) -> Union[Any, None]:
    fromlist = [classname]

    try:
        m = __import__(path, globals(), locals(), fromlist)
    except ImportError:
        raise ImportError("Unable to find module path: {0:s}".format(path))

    try:
        c = getattr(m, classname)
    except AttributeError:
        raise ImportError("Unable to get module attribute: {0:s} with {1:s}".format(path, classname))

    return c


def _get_terminal_size_windows() -> Tuple[int, int]:  # pragma: no cover
    try:
        import struct
        from ctypes import windll, create_string_buffer
        # stdin handle is -10
        # stdout handle is -11
        # stderr handle is -12
        h = windll.kernel32.GetStdHandle(-12)
        csbi = create_string_buffer(22)
        res = windll.kernel32.GetConsoleScreenBufferInfo(h, csbi)
        if res:
            (bufx, bufy, curx, cury, wattr, left, top, right, bottom, maxx, maxy) = struct.unpack("hhhhHhhhhhh",
                                                                                                  csbi.raw)
            sizex = right - left + 1
            sizey = bottom - top + 1
            return sizex, sizey
    except Exception as e:
        print("Unable to get terminal size!")
        print(str(e))
        pass
    return 80, 25


def _get_terminal_size_tput() -> Tuple[int, int]:  # pragma: no cover
    """get terminal width
    src: http://stackoverflow.com/questions/263890/how-do-i-find-the-width-height-of-a-terminal-window
    """
    try:
        import subprocess
        import shlex
        cols = int(subprocess.check_call(shlex.split('tput cols')))
        rows = int(subprocess.check_call(shlex.split('tput lines')))
        ret = (cols, rows)
        return ret
    except Exception as e:
        print_exception(e)
    return 80, 25


# noinspection PyUnresolvedReferences
def _ioctl_gwinsz(fdin):
    import fcntl
    import termios
    cr_data = None

    try:
        cr_data = struct.unpack('hh', fcntl.ioctl(fdin, termios.TIOCGWINSZ, '1234'))
    except Exception as ex:
        print_exception(ex)

    return cr_data


def _get_terminal_size_linux() -> Tuple[int, int]:  # pragma: no cover
    cr = _ioctl_gwinsz(0) or _ioctl_gwinsz(1) or _ioctl_gwinsz(2)
    if not cr:

        try:
            fd = os.open(os.ctermid(), os.O_RDONLY)
            cr = _ioctl_gwinsz(fd)
            os.close(fd)
        except Exception as ex:
            print_exception(ex)

    if not cr:
        try:
            cr = (os.environ['LINES'], os.environ['COLUMNS'])
        except Exception as ex:
            print_exception(ex)
            return 80, 25

    return int(cr[1]), int(cr[0])


def get_terminal_size() -> Tuple[int, int]:  # pragma: no cover
    current_os = platform.system()
    tuple_xy = (80, 40)
    if current_os == 'Windows':
        tuple_xy = _get_terminal_size_windows()

        # needed for window's python in cygwin's xterm!
        if tuple_xy is None:
            tuple_xy = _get_terminal_size_tput()

    if current_os in ['Linux', 'Darwin'] or current_os.startswith('CYGWIN'):
        tuple_xy = _get_terminal_size_linux()

    # default value
    if tuple_xy is None:
        tuple_xy = (0, 0)
    return tuple_xy


def print_exception(e: Exception):
    message1 = "An exception of type {0:s} occurred.".format(type(e).__name__)
    message2 = "Arguments:\n{0!r}".format(e.args)
    print(message1)
    print(message2)
    return


def execute(name: str, command: list, data: list = None, callback: Any = None) -> Tuple[bool, int]:
    try:
        proc = subprocess.Popen(command, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
    except Exception as e:
        if callback is not None:
            callback.exception(e)
            callback.traceback()
        return False, 0

    for _line in proc.stdout:
        line = _line.decode('utf-8')

        line = line.replace("\r\n", "")
        line = line.replace("\n", "")

        if data is not None:
            data.append(line)

        if callback is None:
            continue

        callback.inform(name, line)

    proc.wait()

    returncode = proc.returncode

    if callback is not None:
        callback.inform(name, str(proc.returncode))

    return True, returncode



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#
#    Copyright (C) 2017, Kai Raphahn <kai.raphahn@laburec.de>
#

import sqlite3
from dataclasses import dataclass
from typing import Optional, List, Union

import bbutil
from bbutil.database.sqlite.types import Execute
from bbutil.database.sqlite.manager import Connection
from bbutil.database.types import Data

__all__ = [
    "types",
    "manager",

    "SQLite"
]


@dataclass
class SQLite(object):

    name: str = ""
    manager: Optional[Connection] = None
    use_memory: bool = False
    use_scrict: bool = False
    filename: str = ""

    def prepare(self):
        if self.manager is not None:
            return

        self.manager: Connection = Connection()
        self.manager.setup(use_memory=self.use_memory, filename=self.filename)
        return

    @staticmethod
    def check_minmal_version(major: int, minor: int, patch: int = 0) -> bool:
        _info = sqlite3.sqlite_version_info

        _version = "{0:d}.{1:d}.{2:d}".format(major, minor, patch)
        _error1 = "Version check failed!"
        _error2 = "Minimum version requierd is {0:s}, current version is {1:s}!".format(_version,
                                                                                        sqlite3.sqlite_version)

        _major = int(_info[0])
        _minor = int(_info[1])
        _patch = int(_info[2])

        if _major < major:
            bbutil.log.error(_error1)
            bbutil.log.error(_error2)
            return False

        if _minor < minor:
            bbutil.log.error(_error1)
            bbutil.log.error(_error2)
            return False

        if _patch < patch:
            bbutil.log.error(_error1)
            bbutil.log.error(_error2)
            return False

        return True

    def check(self, table_name: str) -> int:
        _check = self.manager.connect()
        if _check is False:
            return False

        _connection = self.manager.connection
        c = _connection.cursor()
        command = "SELECT name FROM sqlite_master WHERE type='table' AND name='{0:s}';".format(table_name)

        bbutil.log.debug1(self.name, "Check for table: {0:s}".format(table_name))

        try:
            c.execute(command)
        except sqlite3.OperationalError as e:
            bbutil.log.error("Unable to check for table: {0:s}".format(table_name))
            bbutil.log.exception(e)
            self.manager.abort()
            return False

        _res = True
        result = c.fetchone()
        if result is None:
            _res = False

        _check = self.manager.release()
        if _check is False:
            return False

        return _res

    def _count_table(self, table_name: str) -> int:
        c = self.manager.cursor()
        command = "SELECT count(*) FROM {0:s};".format(table_name)

        try:
            c.execute(command)
        except sqlite3.OperationalError as e:
            bbutil.log.error("Unable to count rows: {0:s}".format(table_name))
            bbutil.log.exception(e)
            self.manager.abort()
            return -1

        result = c.fetchall()
        (count,) = result[0]

        bbutil.log.debug1(self.name, "Count table: {0:s}, {1:d}".format(table_name, count))
        return count

    def count(self, table_name: str) -> int:
        _check = self.manager.connect()
        if _check is False:
            return -1

        _count = self._count_table(table_name)
        if _count == -1:
            return -1

        _check = self.manager.release()
        if _check is False:
            return -1

        return _count

    def check_table(self, table_name: str, connect: bool = True) -> bool:
        if connect is True:
            _check = self.manager.connect()
            if _check is False:
                return False

        _connection = self.manager.connection
        c = _connection.cursor()
        command = "SELECT name FROM sqlite_master WHERE type='table' AND name='{0:s}';".format(table_name)

        bbutil.log.debug1(self.name, "Check for table: {0:s}".format(table_name))

        try:
            c.execute(command)
        except sqlite3.OperationalError as e:
            bbutil.log.error("Unable to check for table: {0:s}".format(table_name))
            bbutil.log.exception(e)
            if connect is True:
                self.manager.release()
            return False

        result = c.fetchone()
        if result is None:
            if connect is True:
                self.manager.release()
            return False

        if connect is True:
            _check = self.manager.release()
            if _check is False:
                return False
        return True

    def create_table(self, table_name: str, column_list: list, unique_list: list) -> bool:
        _check = self.manager.connect()
        if _check is False:
            return False

        _connection = self.manager.connection
        c = _connection.cursor()

        _columns = ""

        for _line in column_list:
            if _columns == "":
                _columns = _line
            else:
                _columns = "{0:s}, {1:s}".format(_columns, _line)

        _constraint = ""
        if len(unique_list) > 0:
            _uniques = ", ".join(unique_list)
            _constraint = ", CONSTRAINT constraint_{0:s} UNIQUE ({1:s})".format(table_name, _uniques)

        command = 'CREATE TABLE "{0:s}" ({1:s}{2:s})'.format(table_name, _columns, _constraint)

        # if self.use_scrict is True:
        #     command = 'CREATE TABLE "{0:s}" ({1:s}{2:s}) STRICT'.format(table_name, _columns, _constraint)

        try:
            c.execute(command)
        except sqlite3.OperationalError as e:
            bbutil.log.error("Unable to create table: {0:s}".format(table_name))
            bbutil.log.exception(e)
            print(command)
            self.manager.abort()
            return False

        bbutil.log.debug1(self.name, "Create table: {0:s}".format(table_name))

        _check = self.manager.commit()
        if _check is False:
            self.manager.release()
            return False

        _check = self.manager.release()
        if _check is False:
            return False
        return True

    def get_scheme(self, table_name: str) -> Optional[list]:
        _check = self.manager.connect()
        if _check is False:
            return None

        _connection = self.manager.connection
        c = _connection.cursor()

        command = "SELECT name, type FROM pragma_table_info('{0:s}')".format(table_name)

        try:
            c.execute(command)
        except sqlite3.OperationalError as e:
            bbutil.log.error("Unable to create table: {0:s}".format(table_name))
            bbutil.log.exception(e)
            print(command)
            self.manager.abort()
            return None

        _fetchlist = []

        for _data in c:
            _fetchlist.append(_data)

        _check = self.manager.release()
        if _check is False:
            return None

        if len(_fetchlist) == 0:
            return None

        return _fetchlist

    def _add_columns(self, table_name: str, column_data: str) -> bool:
        _connection = self.manager.connection
        c = _connection.cursor()

        command = 'ALTER TABLE "{0:s}" ADD COLUMN {1:s}'.format(table_name, column_data)

        try:
            c.execute(command)
        except sqlite3.OperationalError as e:
            bbutil.log.error("Unable to alter table: {0:s}".format(table_name))
            bbutil.log.exception(e)
            print(command)
            self.manager.abort()
            return False
        return True

    def add_columns(self, table_name: str, column_list: list) -> bool:
        _check = self.manager.connect()
        if _check is False:
            return False

        _columns = ""

        for _line in column_list:
            _check = self._add_columns(table_name, _line)
            if _check is False:
                return False

        bbutil.log.debug1(self.name, "Alter table: {0:s}".format(table_name))

        _check = self.manager.release()
        if _check is False:
            return False
        return True

    def _drop_columns(self, table_name: str, column_data: str) -> bool:
        _connection = self.manager.connection
        c = _connection.cursor()

        command = 'ALTER TABLE "{0:s}" DROP COLUMN {1:s};'.format(table_name, column_data)

        try:
            c.execute(command)
        except sqlite3.OperationalError as e:
            bbutil.log.error("Unable to alter table: {0:s}".format(table_name))
            bbutil.log.exception(e)
            print(command)
            self.manager.abort()
            return False
        return True

    def drop_columns(self, table_name: str, column_list: list) -> bool:
        _check = self.check_minmal_version(3, 35, 0)
        if _check is False:
            return False

        _check = self.manager.connect()
        if _check is False:
            return False

        _columns = ""

        for _line in column_list:
            _check = self._drop_columns(table_name, _line)
            if _check is False:
                return False

        bbutil.log.debug1(self.name, "Alter table: {0:s}".format(table_name))

        _check = self.manager.release()
        if _check is False:
            return False
        return True

    def rename_table(self, table_name: str, new_name: str) -> bool:
        _check = self.manager.connect()
        if _check is False:
            return False

        _connection = self.manager.connection
        c = _connection.cursor()

        command = 'ALTER TABLE "{0:s}" RENAME TO "{1:s}";'.format(table_name, new_name)

        try:
            c.execute(command)
        except sqlite3.OperationalError as e:
            bbutil.log.error("Unable to rename table: {0:s}".format(table_name))
            bbutil.log.exception(e)
            print(command)
            self.manager.abort()
            return False

        bbutil.log.debug1(self.name, "Rename table: {0:s}".format(table_name))

        _check = self.manager.release()
        if _check is False:
            return False
        return True

    def rename_column(self, table_name: str, column_name: str, new_name: str) -> bool:
        _check = self.check_minmal_version(3, 25, 0)
        if _check is False:
            return False

        print(sqlite3.sqlite_version_info)

        _check = self.manager.connect()
        if _check is False:
            return False

        _connection = self.manager.connection
        c = _connection.cursor()

        command = 'ALTER TABLE "{0:s}" RENAME COLUMN "{1:s}" TO "{2:s}"'.format(table_name,
                                                                                column_name,
                                                                                new_name)

        try:
            c.execute(command)
        except sqlite3.OperationalError as e:
            bbutil.log.error("Unable to rename column: {0:s}".format(table_name))
            bbutil.log.exception(e)
            print(command)
            self.manager.abort()
            return False

        bbutil.log.debug1(self.name, "Rename column: {0:s}".format(table_name))

        _check = self.manager.release()
        if _check is False:
            return False
        return True

    @staticmethod
    def _single_execute(table_name: str, names: list, data: Data) -> Optional[Execute]:
        _data = []
        _names = ", ".join(names)
        _placeholder = ", ".join(['?'] * len(names))

        sql = 'INSERT INTO "{0:s}" ({1:s}) VALUES ({2:s});'.format(table_name, _names, _placeholder)

        for _line in names:
            try:
                _value = getattr(data, _line)
            except AttributeError as e:
                bbutil.log.exception(e)
                bbutil.log.error("Data format does not fit database table!")
                return None
            _data.append(_value)
        _execute = Execute(sql=sql, data=_data)
        return _execute

    @staticmethod
    def _many_execute(table_name: str, names: list, data_list: List[Data]) -> Optional[Execute]:
        _data = []
        _length = len(data_list)
        _names = ", ".join(names)
        _placeholder = ", ".join(['?'] * len(names))

        sql = 'INSERT OR IGNORE INTO "{0:s}" ({1:s}) VALUES ({2:s});'.format(table_name, _names, _placeholder)

        for _item in data_list:
            _value = []
            for _line in names:
                try:
                    _ret = getattr(_item, _line)
                except AttributeError as e:
                    bbutil.log.exception(e)
                    bbutil.log.error("Data format does not fit database table!")
                    return None
                _value.append(_ret)
            _data.append(_value)

        _execute = Execute(sql=sql, data=_data)
        return _execute

    def _insert(self, table_name: str, names: list, data: Union[Data, List[Data]]) -> int:
        c = self.manager.cursor()

        _is_many = True

        if type(data) is Data:
            _is_many = False

        if _is_many is False:
            _execute = self._single_execute(table_name, names, data)
        else:
            _execute = self._many_execute(table_name, names, data)

        if _execute is None:
            return -1

        if _is_many is True:
            command = c.executemany
        else:
            command = c.execute

        try:
            command(_execute.sql, _execute.data)
        except sqlite3.InterfaceError as e:
            bbutil.log.exception(e)
            bbutil.log.error("One or more values is an invalid format!")
            bbutil.log.error("SQL:  " + str(_execute.sql))
            bbutil.log.error("DATA: " + str(_execute.data))
            return -1
        except OverflowError as e:
            bbutil.log.exception(e)
            bbutil.log.error("One or more values is too large!")
            bbutil.log.error("SQL:  " + str(_execute.sql))
            bbutil.log.error("DATA: " + str(_execute.data))
            return -1
        except sqlite3.IntegrityError:
            return -1
        except Exception as e:
            bbutil.log.exception(e)
            bbutil.log.error("SQL:  " + str(_execute.sql))
            bbutil.log.error("DATA: " + str(_execute.data))
            return -1

        _counter = c.rowcount

        if _counter > 0:
            _check = self.manager.commit()
            if _check is False:
                return -1

        return _counter

    @staticmethod
    def _get_chunk_size(max_intervall: int) -> int:
        interval = 1

        if max_intervall > 500:
            interval = 5

        if max_intervall > 1000:
            interval = 10

        if max_intervall > 5000:
            interval = 50

        if max_intervall > 10000:
            interval = 100

        if max_intervall > 20000:
            interval = 200

        if max_intervall > 50000:
            interval = 500

        return interval

    @staticmethod
    def _split_list(data_list: List[Data], chunk_size: int) -> list:
        chunked_list = []
        for i in range(0, len(data_list), chunk_size):
            chunked_list.append(data_list[i:i + chunk_size])

        return chunked_list

    def _insert_list(self, table_name: str, names: list, data_list: List[Data]) -> int:
        _chunk_size = self._get_chunk_size(len(data_list))
        _split_list = self._split_list(data_list, _chunk_size)
        _max = len(_split_list) + 1

        _progress = bbutil.log.progress(_max)

        _counter = 0
        _stored = 0

        for _item_list in _split_list:
            _counter += len(_item_list)
            _result = self._insert(table_name, names, _item_list)
            if _result == -1:
                bbutil.log.clear()
                return -1

            _stored += _result

            _progress.inc()

        bbutil.log.clear()

        if _counter != _stored:
            bbutil.log.warn(self.name, "Entries {0:d}, Stored {1:d}".format(_counter, _stored))
        else:
            bbutil.log.inform(self.name, "Stored {0:d}".format(_counter))

        return _stored

    def insert(self, table_name: str, names: list, data: Union[Data, List[Data]]) -> int:
        _check = self.manager.connect()
        if _check is False:
            return -1

        if type(data) is list:
            count = self._insert_list(table_name, names, data)
        else:
            count = self._insert(table_name, names, data)

        if count == -1:
            self.manager.abort()
            return -1

        _check = self.manager.release()
        if _check is False:
            return -1

        return count

    def update(self, table_name: str, names: list, data: Data, sql_filter: str, filter_value=None) -> bool:
        _check = self.manager.connect()
        if _check is False:
            return False

        c = self.manager.cursor()

        _sets = []
        for _name in names:
            _line = "{0:s} = ?".format(_name)
            _sets.append(_line)

        _data = []
        _names = ", ".join(_sets)

        sql = 'UPDATE "{0:s}" SET {1:s} WHERE {2:s};'.format(table_name, _names, sql_filter)

        for _line in names:
            _value = getattr(data, _line)
            _data.append(_value)

        if filter_value is not None:
            _data.append(filter_value)

        try:
            c.execute(sql, _data)
        except sqlite3.IntegrityError:
            return False
        except sqlite3.OperationalError as e:
            bbutil.log.error("SQL:  " + str(sql))
            bbutil.log.error("DATA: " + str(_data))
            bbutil.log.exception(e)
            return False
        except OverflowError as e:
            bbutil.log.exception(e)
            bbutil.log.error("SQL:  " + str(sql))
            bbutil.log.error("DATA: " + str(_data))
            return False

        _check = self.manager.commit()
        if _check is False:
            return False

        _check = self.manager.release()
        if _check is False:
            return False

        return True

    @staticmethod
    def _select_execute(cursor: sqlite3.Cursor, command: str, data: list):
        if len(data) == 0:
            cursor.execute(command)
        else:
            cursor.execute(command, data)
        return

    def select(self, table_name: str, names: list, sql_filter: str, data: list) -> Optional[list]:
        _check = self.manager.connect()
        if _check is False:
            return None

        c = self.manager.cursor()

        _selector = "*"
        if len(names) != 0:
            _selector = ", ".join(names)

        command = "SELECT {0:s} FROM {1:s};".format(_selector, table_name)

        if sql_filter != "":
            command = "SELECT {0:s} FROM {1:s} WHERE {2:s};".format(_selector, table_name, sql_filter)

        bbutil.log.debug1(table_name, command)

        try:
            self._select_execute(c, command, data)
        except sqlite3.OperationalError as e:
            bbutil.log.error("Unable to search table: {0:s}".format(table_name))
            bbutil.log.exception(e)
            bbutil.log.error("SQL:  " + str(command))
            bbutil.log.error("DATA: " + str(data))
            return None
        except OverflowError as e:
            bbutil.log.error("Unable to search table due to overflow: {0:s}".format(table_name))
            bbutil.log.exception(e)
            bbutil.log.error("SQL:  " + str(command))
            bbutil.log.error("DATA: " + str(data))
            return None

        _fetchlist = []

        for _data in c:
            _fetchlist.append(_data)

        _check = self.manager.release()
        if _check is False:
            return None

        return _fetchlist



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#
#    Copyright (C) 2017, Kai Raphahn <kai.raphahn@laburec.de>
#

import sqlite3
import os
import unittest
import unittest.mock as mock

from unittest.mock import Mock

from bbutil.database.sqlite.manager import Connection
from bbutil.utils import full_path

from tests.helper.sqlite import sqlite_operational_error, mock_operational_error

from tests.helper import set_log

__all__ = [
    "TestSQLiteManager"
]


class TestSQLiteManager(unittest.TestCase):
    """Testing class for locking module."""

    @staticmethod
    def _clean(filename: str, connection: sqlite3.Connection = None):
        if connection is not None:
            connection.close()

        if os.path.exists(filename) is True:
            os.remove(filename)
        return

    def setUp(self):
        set_log()
        return

    def test_setup_01(self):
        _testfile = full_path("{0:s}/test.sqlite".format(os.getcwd()))
        _name = "Test"

        if os.path.exists(_testfile) is True:
            os.remove(_testfile)

        _connection = Connection()
        _connection.setup(filename=_testfile, use_memory=False)

        self.assertEqual(_connection.filename, _testfile)
        self.assertEqual(_connection.use_memory, False)
        self.assertIsNotNone(_connection._lock)
        return

    def test_setup_02(self):
        _testfile = full_path("{0:s}/test.sqlite".format(os.getcwd()))
        _name = "Test"

        if os.path.exists(_testfile) is True:
            os.remove(_testfile)

        _connection = Connection()
        _connection.setup(use_memory=True)

        self.assertEqual(_connection.filename, "")
        self.assertEqual(_connection.use_memory, True)
        self.assertIsNotNone(_connection._lock)
        return

    def test_abort_01(self):
        _testfile = full_path("{0:s}/test.sqlite".format(os.getcwd()))
        _name = "Test"

        if os.path.exists(_testfile) is True:
            os.remove(_testfile)

        _connection = Connection()

        _connection.abort()
        self.assertIsNone(_connection._lock)

        _connection.setup(filename=_testfile, use_memory=False)

        _connection._lock.acquire()
        _connection.abort()
        return

    def test_reset_01(self):
        _testfile = full_path("{0:s}/test.sqlite".format(os.getcwd()))
        _name = "Test"

        if os.path.exists(_testfile) is True:
            os.remove(_testfile)

        _connection = Connection()
        _connection.setup(filename=_testfile, use_memory=False)

        self.assertIsNotNone(_connection._lock)

        _check = _connection.connect()

        _con = _connection.connection

        self.assertIsNotNone(_connection.connection)

        _connection.reset()
        self.assertIsNone(_connection.connection)

        self._clean(_testfile, _con)
        return

    def test_connect_01(self):
        _testfile = full_path("{0:s}/test.sqlite".format(os.getcwd()))

        if os.path.exists(_testfile) is True:
            os.remove(_testfile)

        _connection = Connection()
        _connection.setup(filename=_testfile, use_memory=False)

        self.assertEqual(_connection.filename, _testfile)
        self.assertEqual(_connection.use_memory, False)
        self.assertIsNotNone(_connection._lock)

        _check = _connection.connect()
        c = _connection.cursor()
        self.assertTrue(_check)
        self.assertIsNotNone(_connection.connection)
        self.assertIsNotNone(c)

        _check = _connection.release()
        self.assertTrue(_check)
        return

    @mock.patch('sqlite3.connect', new=mock_operational_error)
    def test_connect_02(self):
        _testfile = full_path("{0:s}/test.sqlite".format(os.getcwd()))

        if os.path.exists(_testfile) is True:
            os.remove(_testfile)

        _connection = Connection()
        _connection.setup(filename=_testfile, use_memory=False)

        self.assertEqual(_connection.filename, _testfile)
        self.assertEqual(_connection.use_memory, False)
        self.assertIsNotNone(_connection._lock)

        _check = _connection.connect()
        c = _connection.cursor()
        self.assertFalse(_check)
        self.assertIsNone(_connection.connection)
        self.assertIsNone(c)

        _check = _connection.release()
        self.assertFalse(_check)
        return

    def test_connect_03(self):
        _connection = Connection()
        _connection.setup(use_memory=True)

        self.assertTrue(_connection.use_memory)
        self.assertIsNotNone(_connection._lock)

        _check = _connection.connect()
        c = _connection.cursor()
        self.assertTrue(_check)
        self.assertIsNotNone(_connection.connection)
        self.assertIsNotNone(c)

        _check = _connection.release()
        self.assertTrue(_check)
        return

    @mock.patch('sqlite3.connect', new=mock_operational_error)
    def test_connect_04(self):
        _connection = Connection()
        _connection.setup(use_memory=True)

        self.assertTrue(_connection.use_memory)
        self.assertIsNotNone(_connection._lock)

        _check = _connection.connect()
        c = _connection.cursor()
        self.assertFalse(_check)
        self.assertIsNone(_connection.connection)
        self.assertIsNone(c)

        _check = _connection.release()
        self.assertFalse(_check)
        return

    @mock.patch('sqlite3.connect', new=mock_operational_error)
    def test_connect_05(self):
        _testfile = full_path("{0:s}/test.sqlite".format(os.getcwd()))

        if os.path.exists(_testfile) is True:
            os.remove(_testfile)

        _connection = Connection()
        _connection.setup(filename=_testfile, use_memory=False)

        self.assertEqual(_connection.filename, _testfile)
        self.assertEqual(_connection.use_memory, False)
        self.assertIsNotNone(_connection._lock)

        _connection._connection = Mock()

        _check = _connection.connect()
        self.assertFalse(_check)
        return

    def test_release_01(self):
        _testfile = full_path("{0:s}/test.sqlite".format(os.getcwd()))

        if os.path.exists(_testfile) is True:
            os.remove(_testfile)

        _connection = Connection()
        _connection.setup(filename=_testfile, use_memory=False)

        _check = _connection.connect()
        self.assertTrue(_check)

        _check = _connection.release()
        self.assertTrue(_check)
        return

    def test_release_02(self):
        _testfile = full_path("{0:s}/test.sqlite".format(os.getcwd()))

        if os.path.exists(_testfile) is True:
            os.remove(_testfile)

        _connection = Connection()
        _connection.setup(filename=_testfile, use_memory=False)

        _check = _connection.release()
        self.assertFalse(_check)
        return

    def test_release_03(self):
        _testfile = full_path("{0:s}/test.sqlite".format(os.getcwd()))

        if os.path.exists(_testfile) is True:
            os.remove(_testfile)

        _connection = Connection()
        _connection._connection = Mock()

        _check = _connection.release()
        self.assertFalse(_check)
        return

    def test_release_04(self):
        _testfile = full_path("{0:s}/test.sqlite".format(os.getcwd()))

        if os.path.exists(_testfile) is True:
            os.remove(_testfile)

        _connection = Connection()
        _connection.setup(filename=_testfile, use_memory=False)

        _connection._connection = Mock()
        _connection._connection.close = Mock(side_effect=sqlite_operational_error)

        _check = _connection.release()
        self.assertFalse(_check)
        return

    def test_commit_01(self):
        _testfile = full_path("{0:s}/test.sqlite".format(os.getcwd()))

        if os.path.exists(_testfile) is True:
            os.remove(_testfile)

        _connection = Connection()
        _connection.setup(filename=_testfile, use_memory=False)

        _check = _connection.connect()
        self.assertTrue(_check)

        _check = _connection.commit()
        self.assertTrue(_check)

        _check = _connection.release()
        self.assertTrue(_check)
        return

    def test_commit_02(self):
        _testfile = full_path("{0:s}/test.sqlite".format(os.getcwd()))

        if os.path.exists(_testfile) is True:
            os.remove(_testfile)

        _connection = Connection()
        _connection.setup(filename=_testfile, use_memory=False)

        _check = _connection.commit()
        self.assertFalse(_check)
        return

    def test_commit_03(self):
        _testfile = full_path("{0:s}/test.sqlite".format(os.getcwd()))

        if os.path.exists(_testfile) is True:
            os.remove(_testfile)

        _connection = Connection()
        _connection.setup(filename=_testfile, use_memory=False)

        _connection._connection = Mock()
        _connection._connection.commit = Mock(side_effect=sqlite_operational_error)

        _check = _connection.commit()
        self.assertFalse(_check)
        return



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#
#    Copyright (C) 2017, Kai Raphahn <kai.raphahn@laburec.de>
#


import sys
import bbutil.lang.parser
import bbutil.lang.parser.pyfile

from bbutil.logging import Logging
from bbutil.lang.parser import Parser, Command

log: Logging = Logging()


if __name__ == '__main__':

    bbutil.lang.parser.set_log(log)
    bbutil.lang.parser.pyfile.set_log(log)
    log.setup(app="make-lang.py", level=0)

    console = log.get_writer("console")
    console.setup(text_space=12, error_index=["ERROR", "EXCEPTION"])
    log.register(console)
    _check = log.open()
    if _check is False:
        sys.exit(1)

    main = Parser()

    _check = main.init(sys.argv[1:])
    if _check is False:
        sys.exit(1)

    _check = main.parse()
    if _check is False:
        sys.exit(1)

    _check = main.store(command=Command.generate)
    if _check is False:
        sys.exit(1)

    _check = main.store(command=Command.update)
    if _check is False:
        sys.exit(1)

    _check = main.store(command=Command.copy)
    if _check is False:
        sys.exit(1)

    _check = main.store(command=Command.compile)
    if _check is False:
        sys.exit(1)



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#
#    Copyright (C) 2017, Kai Raphahn <kai.raphahn@laburec.de>
#

import unittest

import unittest.mock as mock

from tests.helper import set_log
from tests.helper.module import config_testone, config_testone_xx

from bbutil.app.module import Module

__all__ = [
    "TestModule"
]

oserror = OSError("Something strange did happen!")
mock_oserror = mock.Mock(side_effect=oserror)
mock_remove = mock.Mock()


class TestModule(unittest.TestCase):
    """Testing class for locking module."""

    def setUp(self):
        set_log()
        return

    def test_init_01(self):
        _module = Module(**config_testone)

        self.assertEqual(_module.command, "test01")
        self.assertEqual(_module.desc, "the first test")
        self.assertEqual(_module.count, 0)

        _check = _module.load()
        self.assertTrue(_check)
        self.assertEqual(_module.count, 2)

        _worker = _module.get_worker("Act01")
        self.assertIsNotNone(_worker)

        _worker = _module.get_worker("Act00")
        self.assertIsNone(_worker)
        return

    def test_init_02(self):
        _module = Module(**config_testone_xx)

        _check = _module.load()
        self.assertFalse(_check)
        return



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#
#    Copyright (C) 2019, Kai Raphahn <kai.raphahn@laburec.de>
#

from setuptools import setup, find_packages
import bbutil

with open("README.md", "r") as fh:
    long_description = fh.read()

packages = find_packages(where=".", exclude=["tests", "tests.logging", "tests.lang"])

setup(
    name=bbutil.__name__,
    license=bbutil.__license__,
    version=bbutil.__version__,
    description=bbutil.__description__,
    author=bbutil.__author__,
    author_email=bbutil.__email__,
    include_package_data=True,
    long_description=long_description,
    long_description_content_type="text/markdown",
    scripts=[
        'run-tests.py',
        'make-lang.py'
    ],
    url='https://github.com/TheUncleKai/bbutils',
    packages=packages,
    python_requires='>=3.6',
    classifiers=[
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: Apache Software License',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Libraries'
    ],
    install_requires=[
        'colorama'
    ]
)



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#
#    Copyright (C) 2017, Kai Raphahn <kai.raphahn@laburec.de>
#

from typing import Any, Tuple
from enum import Enum


__all__ = [
    "Type",
    "Data",
    "Convert"
]


class Type(Enum):
    """Entry types."""

    Float = float
    """represent an integer."""

    Number = int
    """represent an integer."""

    String = str
    """represent a string."""

    Boolean = bool
    """represent a boolean."""

    Array = list
    """represent an array."""

    Tuple = tuple
    """represent an tuple."""

    Object = dict
    """represent a dictionary type object."""

    Null = None
    """represent none."""


class Data(object):

    def __init__(self, **kwargs):

        self._id = kwargs.get("id", "")
        keys = kwargs.get("keys", None)
        values = kwargs.get("values", None)

        if (keys is None) or (values is None):
            return

        for (key, value) in zip(keys, values):
            self.__dict__[key] = value
        return

    def __str__(self) -> str:
        return self._id


class Convert(object):

    def __init__(self):
        return

    @staticmethod
    def check_name(name: str) -> bool:
        white_list = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789_"

        for char in name:
            if char in white_list:
                continue
            else:
                raise ValueError("Invald character {0:s} in name: {1:s}".format(char, name))

        return True

    def _convert_list(self, data: list) -> list:
        """Convert dict or die.

        :param data: data item.
        :type data: list

        :return: list object or None.
        """

        values = []

        for item in data:
            value, check = self._convert("", item)
            if check is True:
                values.append(value)

        return values

    def _convert_dict(self, name: str, data: dict) -> Any:
        """Convert dict or die.

        :param name: item name.
        :type name: str

        :param data: data item.
        :type data: dict

        :return: data object or None.
        """

        names = []
        values = []

        for key in list(data):
            item = data[key]
            check = self.check_name(key)
            if check is False:
                return None
            value, check = self._convert(key, item)
            if check is False:
                return None

            names.append(key)
            values.append(value)

        if (len(names) == 0) or (len(values) == 0):  # pragma: no cover
            return None

        config = Data(id=name, keys=names, values=values)
        return config

    def _convert(self, itemid: str, data: Any) -> Tuple[Any, bool]:
        """Convert item or die.

        :param itemid: root id.
        :type itemid: str

        :param data: item data.

        :return: data object or None and if conversion was successfull
        :rtype: tuple
        """

        type_data = type(data)

        check_object = type_data is Type.Object.value
        check_array = type_data is Type.Array.value
        check_tuple = type_data is Type.Tuple.value

        check_string = type_data is Type.String.value
        check_number = type_data is Type.Number.value
        check_float = type_data is Type.Float.value
        check_boolean = type_data is Type.Boolean.value
        check_null = data is Type.Null.value

        value = None
        valid = False

        if check_object is True:
            value = self._convert_dict(itemid, data)
            valid = True

        if check_array is True:
            value = self._convert_list(data)
            valid = True

        if check_tuple is True:
            value = self._convert_list(data)
            valid = True

        if check_string is True:
            value = data
            valid = True

        if check_number is True:
            value = data
            valid = True

        if check_float is True:
            value = data
            valid = True

        if check_boolean is True:
            value = data
            valid = True

        if check_null is True:
            valid = True

        if (value is None) and (check_object is False) and (check_null is False):
            raise ValueError("Invalid data type for {0:s}".format(itemid))

        return value, valid

    def parse(self, data: Any) -> Any:
        """Parse given data.

        :return: converted data or none.
        """
        ret, _ = self._convert("", data)
        return ret



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#
#    Copyright (C) 2017, Kai Raphahn <kai.raphahn@laburec.de>
#

import os
import unittest
import unittest.mock as mock

from bbutil.database import Database

from tests.helper import set_log
from tests.helper.database import TestData

from tests.helper.sqlite import (mock_operational_error, sqlite_operational_error)

__all__ = [
    "TestDatabase"
]


class TestDatabase(unittest.TestCase):
    """Testing class for locking module."""

    def setUp(self):
        set_log()
        return

    @staticmethod
    def _clean(database: Database):
        if database.sqlite is None:
            return

        _con = None

        if database.sqlite.manager is not None:
            _con = database.sqlite.manager.connection

        if _con is not None:
            _con.close()

        if os.path.exists(database.sqlite.filename) is True:
            os.remove(database.sqlite.filename)
        return

    def assertHasAttr(self, obj, intended_attr):
        _testBool = hasattr(obj, intended_attr)

        self.assertTrue(_testBool, msg=f'obj lacking an attribute. {obj=}, {intended_attr=}')
        return

    def test_start_01(self):
        _filename = "{0:s}/test.sqlite".format(os.getcwd())

        _database = TestData(filename=_filename)

        _check1 = _database.start()
        self.assertIsNotNone(_database.table01)
        self.assertIsNotNone(_database.table02)

        self._clean(_database)
        return

    def test_start_02(self):
        _database = TestData(filename="")

        _check1 = _database.start()
        self.assertFalse(_check1)

        self._clean(_database)
        return

    def test_start_03(self):
        _filename = "{0:s}/test.sqlite".format(os.getcwd())
        _database = TestData(filename=_filename, prepare_fail=True)

        _check1 = _database.start()

        self.assertFalse(_check1)
        self._clean(_database)
        return

    @mock.patch('sqlite3.connect', new=mock_operational_error)
    def test_start_04(self):
        _filename = "{0:s}/test.sqlite".format(os.getcwd())
        _database = TestData(filename=_filename)

        _check1 = _database.start()

        self.assertFalse(_check1)
        self._clean(_database)
        return

    def test_start_05(self):
        _filename = "{0:s}/testdata/database/test_database.sqlite".format(os.getcwd())
        _database = TestData(filename=_filename)

        _check1 = _database.start()
        self.assertIsNotNone(_database.table01)
        self.assertIsNotNone(_database.table02)
        return

    def test_get_table_01(self):
        _filename = "{0:s}/testdata/database/test_database.sqlite".format(os.getcwd())
        _database = TestData(filename=_filename)

        _check1 = _database.start()

        _table1 = _database.get_table("tester01")
        _table2 = _database.get_table("tester01XXXX")

        self.assertTrue(_check1)
        self.assertIsNotNone(_database.table01)
        self.assertIsNotNone(_database.table02)
        self.assertIsNotNone(_table1)
        self.assertIsNone(_table2)
        return

    def test_store_01(self):
        _filename = "{0:s}/test.sqlite".format(os.getcwd())

        if os.path.exists(_filename) is True:
            os.remove(_filename)

        _database = TestData(filename=_filename)

        _check1 = _database.start()
        self.assertIsNotNone(_database.table01)
        self.assertIsNotNone(_database.table02)

        _data = _database.table01.new_data()
        _data.use_test = False
        _data.testname = "01"
        _data.path = "//"

        _database.table01.add(_data)

        _data = _database.table02.new_data()
        _data.use_test = False
        _data.category = "01"
        _data.testname = "01"
        _data.path = "//"

        _database.table02.add(_data)

        _count3 = _database.store()

        _count1 = _database.table01.check()
        _count2 = _database.table02.check()

        self.assertTrue(_check1)
        self.assertEqual(_count1, 1)
        self.assertEqual(_count2, 1)
        self.assertEqual(_count3, 2)
        self._clean(_database)
        return

    def test_load_01(self):
        _filename = "{0:s}/testdata/database/test_database.sqlite".format(os.getcwd())
        _database = TestData(filename=_filename)

        _check1 = _database.start()
        self.assertIsNotNone(_database.table01)
        self.assertIsNotNone(_database.table02)

        _count = _database.load()

        self.assertTrue(_check1)
        self.assertEqual(_count, 2)
        self.assertEqual(_database.table01.data_count, 1)
        self.assertEqual(_database.table02.data_count, 1)
        return

    def test_clear_01(self):
        _filename = "{0:s}/testdata/database/test_database.sqlite".format(os.getcwd())
        _database = TestData(filename=_filename)

        _check1 = _database.start()
        self.assertIsNotNone(_database.table01)
        self.assertIsNotNone(_database.table02)

        _count = _database.load()

        self.assertTrue(_check1)
        self.assertEqual(_count, 2)
        self.assertEqual(_database.table01.data_count, 1)
        self.assertEqual(_database.table02.data_count, 1)

        _database.clear()
        self.assertEqual(_database.table01.data_count, 0)
        self.assertEqual(_database.table02.data_count, 0)
        return



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#
#    Copyright (C) 2017, Kai Raphahn <kai.raphahn@laburec.de>
#

import sys
import time
import traceback
from threading import Thread
from typing import List, Dict

import bbutil.logging.writer

from bbutil.logging.types import Timer, Message, Writer, Progress
from bbutil.utils import get_attribute

__all__ = [
    "writer",
    "types",

    "Logging"
]

_index = {
    0: ["INFORM", "WARN", "ERROR", "EXCEPTION", "TIMER", "PROGRESS"],
    1: ["INFORM", "DEBUG1", "WARN", "ERROR", "EXCEPTION", "TIMER", "PROGRESS"],
    2: ["INFORM", "DEBUG1", "DEBUG2", "WARN", "ERROR", "EXCEPTION", "TIMER", "PROGRESS"],
    3: ["INFORM", "DEBUG1", "DEBUG2", "DEBUG3", "WARN", "ERROR", "EXCEPTION", "TIMER", "PROGRESS"]
}


class LogState(object):

    def __init__(self):
        self.close: bool = False
        self.open: bool = False
        self.use_thread: bool = False
        self.thread_active: bool = False
        return


class Logging(object):

    def __init__(self):
        self._level: int = 0
        self._app: str = ""
        self._timer_list: List[Timer] = []
        self._timer_counter: int = 0

        self._buffer: List[Message] = []
        self._interval: float = 0.01
        self._index: Dict[int, List[str]] = {}
        self._writer: List[Writer] = []

        self.state: LogState = LogState()
        return

    def __del__(self):
        return

    @property
    def verbose(self) -> int:
        return self._level

    @staticmethod
    def get_writer(name: str) -> Writer:
        c = None

        for item in bbutil.logging.writer.__all__:

            if item != name:
                continue

            path = "bbutil.logging.writer.{0:s}".format(item)
            classname = get_attribute(path, "classname")
            c = get_attribute(path, classname)

        if c is None:
            raise ImportError("Unable to find writer: {0:s}".format(name))

        _writer = c()
        return _writer

    def _process(self, item: Message):
        for _writer in self._writer:
            if (item.level not in _writer.index) and (item.raw is False):
                continue
            _writer.write(item)
        return

    def _run(self):
        self.state.thread_active = True

        while True:
            item_counter = len(self._buffer)

            self._thread = True

            # only quit when buffer is empty
            if (self.state.close is True) and (item_counter == 0):
                self.state.thread_active = False
                break

            time.sleep(self._interval)
            count = len(self._buffer)

            if count == 0:
                continue

            _message = self._buffer.pop(0)
            self._process(_message)
        return

    def register(self, write: Writer):
        self._writer.append(write)
        return

    def append(self, item: Message):
        if self._level not in self._index:
            return

        level_list = self._index[self._level]

        if (item.level not in level_list) and (item.raw is False):
            return

        item.app = self._app

        # select output method: if threaded use buffer for output, if not print directly
        if self.state.use_thread is True:
            self._buffer.append(item)
        else:
            self._process(item)
        return

    def setup(self, **kwargs):
        item = kwargs.get("app", None)
        if item is not None:
            self._app = item

        item = kwargs.get("interval", None)
        if item is not None:
            self._interval = item

        item = kwargs.get("level", None)
        if item is not None:
            self._level = item

        item = kwargs.get("index", None)
        if item is not None:
            self._index = item

        item = kwargs.get("use_thread", None)
        if item is not None:
            self.state.use_thread = item
        return

    def open(self) -> bool:
        if (self.state.use_thread is True) and (self.state.thread_active is True):
            print("A logging thread is already running!")
            return False

        length = len(self._index)
        if length == 0:
            self._index = _index

        if len(self._writer) == 0:
            print("Not output writer selected!")
            return False

        for item in self._writer:
            check = item.open()
            if check is False:
                return False

        self.state.open = True

        if self.state.use_thread is True:
            thread = Thread(target=self._run)
            thread.start()
        return True

    def _close_thread(self):
        if self.state.use_thread is False:
            return

        # wait for thread to close
        while True:
            if self.state.thread_active is False:
                break
            time.sleep(0.01)
        return

    def close(self) -> bool:
        self.state.close = True

        self._close_thread()
        for item in self._writer:
            check = item.close()
            if check is False:
                return False

        return True

    def clear(self):
        for _writer in self._writer:
            _writer.clear()
        return

    def raw(self, content: str):
        _message = Message(content=content, raw=True)
        self.append(_message)
        return

    def inform(self, tag: str, content: str):
        _message = Message(tag=tag, content=content, level="INFORM")
        self.append(_message)
        return

    def warn(self, tag: str, content: str):
        _message = Message(tag=tag, content=content, level="WARN")
        self.append(_message)
        return

    def debug1(self, tag: str, content: str):
        _message = Message(tag=tag, content=content, level="DEBUG1")
        self.append(_message)
        return

    def debug2(self, tag: str, content: str):
        _message = Message(tag=tag, content=content, level="DEBUG2")
        self.append(_message)
        return

    def debug3(self, tag: str, content: str):
        _message = Message(tag=tag, content=content, level="DEBUG3")
        self.append(_message)
        return

    def error(self, content: str):
        _message = Message(content=content, level="ERROR")
        self.append(_message)
        return

    def exception(self, e: Exception):
        content = "An exception of type {0} occurred.".format(type(e).__name__)
        _message = Message(content=content, level="EXCEPTION")
        self.append(_message)

        content = "Arguments:\n{0!r}".format(e.args)
        _message = Message(content=content, level="EXCEPTION")
        self.append(_message)
        return

    def traceback(self):

        ttype, value, tb = sys.exc_info()
        self.error("Uncaught exception")
        self.error("Type:  " + str(ttype))
        self.error("Value: " + str(value))

        lines = traceback.format_tb(tb)
        for line in lines:
            _message = Message(content=line, raw=True)
            self.append(_message)
        return

    def progress(self, limit: int, interval: int = 0) -> Progress:
        _progress = Progress(limit, interval, self.append)
        return _progress

    def timer(self, content: str) -> Timer:
        _timer = Timer(content, self.append)
        return _timer



#!/usr/bin/python3
# coding=utf-8

# Copyright (C) 2020, Siemens Healthcare Diagnostics Products GmbH
# Licensed under the Siemens Inner Source License 1.2, see LICENSE.md.

import abc
import os.path

from dataclasses import dataclass
from abc import ABCMeta

from bbutil.utils import full_path

import bbutil

__all__ = [
    "File",
    "Folder",
    "Base"
]


@dataclass
class Base(metaclass=ABCMeta):

    path: str = ""
    basename: str = ""

    @property
    def exists(self) -> bool:
        _check = os.path.exists(self.fullpath)
        return _check

    @property
    def fullpath(self) -> str:
        _filename = full_path("{0:s}/{1:s}".format(self.path, self.basename))
        return _filename

    @property
    def valid(self) -> bool:
        if self.path == "":
            return False

        if self.basename == "":
            return False
        return True

    @abc.abstractmethod
    def init(self) -> bool:
        pass

    def clear(self):
        self.basename = ""
        self.path = ""

        self.clear_object()
        return

    @property
    def check(self) -> bool:
        if self.valid is False:
            bbutil.log.error("Given filename is not valid!")
            return False

        if self.exists is False:
            bbutil.log.error("Given filename does not exists!")
            return False
        return True

    def open(self, filename: str) -> bool:
        self.basename = os.path.basename(filename)
        self.path = os.path.dirname(filename)

        if self.check is False:
            return False

        _check = self.open_file()
        return _check

    @abc.abstractmethod
    def clear_object(self):
        pass

    @abc.abstractmethod
    def open_file(self) -> bool:
        pass

    @abc.abstractmethod
    def create(self) -> bool:
        pass

    def _remove_file(self) -> bool:
        try:
            os.remove(self.fullpath)
        except OSError as e:
            bbutil.log.exception(e)
            return False

        if self.exists is True:
            bbutil.log.error("Unable to remove file!")
            return False

        return True

    def _remove_folder(self) -> bool:
        try:
            os.rmdir(self.fullpath)
        except OSError as e:
            bbutil.log.exception(e)
            return False

        if self.exists is True:
            bbutil.log.error("Unable to remove file!")
            return False

        return True

    def remove(self) -> bool:
        if self.valid is False:
            return True

        if self.exists is False:
            return True

        if os.path.isdir(self.fullpath) is True:
            _check = self._remove_folder()
        else:
            _check = self._remove_file()

        return _check


class File(Base):

    def init(self) -> bool:
        return True

    def clear_object(self):
        return

    def open_file(self) -> bool:
        return True

    def create(self) -> bool:
        return True


class Folder(Base):

    def init(self) -> bool:
        return True

    def clear_object(self):
        return

    def open_file(self) -> bool:
        return True

    def create(self) -> bool:
        if self.exists is False:
            try:
                os.mkdir(self.fullpath)
            except OSError as e:
                bbutil.log.exception(e)
                return False

        if self.exists is False:
            bbutil.log.error("Unable to create: {0:s}".format(self.fullpath))
            return False
        return True



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#
#    Copyright (C) 2017, Kai Raphahn <kai.raphahn@laburec.de>
#

import os

from shutil import copyfile

import bbutil
from bbutil.logging import Logging
from bbutil.app.manager import ModuleManager
from bbutil.database import SQLite
from bbutil.utils import full_path

__all__ = [
    "config",
    "console",
    "database",
    "execute",
    "file",
    "module",
    "setup",
    "sqlite",
    "table",
    "worker",

    "get_sqlite",
    "copy_sqlite",
    "set_log",
    "set_module",
    "reset_module"
]

_index = {
    0: ["INFORM", "WARN", "ERROR", "EXCEPTION", "TIMER", "PROGRESS"],
    1: ["INFORM", "DEBUG1", "WARN", "ERROR", "EXCEPTION", "TIMER", "PROGRESS"],
    2: ["INFORM", "DEBUG1", "DEBUG2", "WARN", "ERROR", "EXCEPTION", "TIMER", "PROGRESS"],
    3: ["INFORM", "DEBUG1", "DEBUG2", "DEBUG3", "WARN", "ERROR", "EXCEPTION", "TIMER", "PROGRESS"]
}


def get_sqlite(filename: str, path: str = os.getcwd(), clean: bool = False) -> SQLite:
    _testfile = full_path("{0:s}/{1:s}".format(path, filename))
    _name = "Test"

    if (os.path.exists(_testfile) is True) and (clean is True):
        os.remove(_testfile)

    _sqlite = SQLite(filename=_testfile, name="Test")
    return _sqlite


def copy_sqlite(filename: str, path: str = os.getcwd(), clean: bool = False) -> SQLite:
    _sourcefile = full_path("{0:s}/{1:s}".format(path, filename))
    _name = "Test"
    _testfile = "{0:s}/{1:s}".format(os.getcwd(), filename)

    if os.path.exists(_testfile):
        os.remove(_testfile)

    copyfile(_sourcefile, _testfile)

    _sqlite = SQLite(filename=_testfile, name="Test")
    return _sqlite


def set_log():
    if bbutil.log is not None:
        return

    _log = Logging()
    _log.setup(app="Test", level=2, index=_index)

    console = _log.get_writer("console")
    _log.register(console)
    _log.open()

    bbutil.set_log(_log)
    return


_config_modules = [
    {
        "name": "testone",
        "command": "test01",
        "desc": "the first test",
        "workers": [
            {
                "path": "testdata.app.commands.prepact",
                "classname": "Worker01"
            },
            {
                "path": "testdata.app.commands.runact",
                "classname": "Worker02"
            }
        ]
    },
    {
        "name": "testtwo",
        "command": "test02",
        "desc": "the second test",
        "workers": [
            {
                "path": "testdata.app.commands.prepact",
                "classname": "Worker01"
            }
        ]
    },
    {
        "name": "testthree",
        "command": "test03",
        "desc": "the third test",
        "workers": [
            {
                "path": "testdata.app.commands.prepact2",
                "classname": "Worker03"
            }
        ]
    }
]


def set_module():
    if bbutil.module is not None:
        return

    _module = ModuleManager()
    _check = _module.init(_config_modules)

    if _check is False:
        return

    bbutil.set_module(_module)
    return


def reset_module():
    if bbutil.module is None:
        return

    bbutil.set_module(None)
    return



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#
#    Copyright (C) 2017, Kai Raphahn <kai.raphahn@laburec.de>
#

from bbutil.database import SQLite, Table, Types

__all__ = [
    "TestData",

    "get_table_01",
    "get_table_02",
    "get_table_03",
    "get_table_04"
]


def get_table_01(sqlite_object: SQLite) -> Table:
    _table = Table(name="tester01", sqlite=sqlite_object)
    _table.add_column(name="testid", data_type=Types.integer, unique=True, keyword=True)
    _table.add_column(name="use_test", data_type=Types.bool)
    _table.add_column(name="testname", data_type=Types.string)
    _table.add_column(name="path", data_type=Types.string)
    return _table


def get_table_02(sqlite_object: SQLite) -> Table:
    _table = Table(name="tester01", sqlite=sqlite_object)
    _table.add_column(name="testid", data_type=Types.integer, unique=True, keyword=True)
    _table.add_column(name="use_test", data_type=Types.bool)
    _table.add_column(name="testname", data_type=Types.string)
    _table.add_column(name="path", data_type=Types.string)
    _table.add_column(name="file", data_type=Types.string)
    return _table


def get_table_03(name: str, sqlite_object: SQLite) -> Table:
    _table = Table(name=name, sqlite=sqlite_object)
    _table.add_column(name="testid", data_type=Types.integer, primarykey=True)
    _table.add_column(name="use_test", data_type=Types.bool)
    _table.add_column(name="testname", data_type=Types.string)
    _table.add_column(name="path", data_type=Types.string)
    return _table


def get_table_04(sqlite_object: SQLite) -> Table:
    _table = Table(name="tester01", sqlite=sqlite_object)
    _table.add_column(name="testid", data_type=Types.integer, primarykey=True)
    _table.add_column(name="use_test", data_type=Types.bool)
    _table.add_column(name="category", data_type=Types.string, keyword=True)
    _table.add_column(name="testname", data_type=Types.string)
    _table.add_column(name="path", data_type=Types.string)
    return _table


class TestData(object):

    def __init__(self):
        self.testid: int = 0
        self.use_test: bool = False
        self.xcategory: str = ""
        self.testname: str = ""
        self.path: str = ""
        return



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#
#    Copyright (C) 2017, Kai Raphahn <kai.raphahn@laburec.de>
#

__all__ = [
    "Callback"
]


class Callback(object):

    def __init__(self):
        self.start = None
        self.stop = None
        self.prepare = None
        self.run = None
        self.close = None
        self.abort = None
        return

    def set_callback(self, **kwargs):
        _value = kwargs.get("start", None)
        if _value is not None:
            self.start = _value

        _value = kwargs.get("stop", None)
        if _value is not None:
            self.stop = _value

        _value = kwargs.get("prepare", None)
        if _value is not None:
            self.prepare = _value

        _value = kwargs.get("run", None)
        if _value is not None:
            self.run = _value

        _value = kwargs.get("close", None)
        if _value is not None:
            self.close = _value

        _value = kwargs.get("abort", None)
        if _value is not None:
            self.abort = _value
        return

    def do_start(self):
        if self.start is None:
            return
        self.start()
        return

    def do_stop(self):
        if self.stop is None:
            return
        self.stop()
        return

    def do_prepare(self):
        if self.prepare is None:
            return
        self.prepare()
        return

    def do_run(self):
        if self.run is None:
            return
        self.run()
        return

    def do_close(self):
        if self.close is None:
            return
        self.close()
        return

    def do_abort(self):
        if self.abort is None:
            return
        self.abort()
        return
